@@ -31,7 +31,7 @@ standard Python ``list``, ``set`` and ``dict`` interfaces.  These could be
 specified in terms of generic decorator recipes, but are instead hand-tooled for
 increased efficiency.   The targeted decorators occasionally implement
 adapter-like behavior, such as mapping bulk-set methods (``extend``, ``update``,
-``__setslice``, etc.) into the series of atomic mutation events that the ORM
+``__setslice__``, etc.) into the series of atomic mutation events that the ORM
 requires.
 
 The targeted decorators are used internally for automatic instrumentation of
@@ -95,10 +95,14 @@ The owning object and InstrumentedCollectionAttribute are also reachable
 through the adapter, allowing for some very sophisticated behavior.
 """
 
-import copy, inspect, sys, weakref
+import copy
+import inspect
+import sets
+import sys
+import weakref
 
 from sqlalchemy import exceptions, schema, util as sautil
-from sqlalchemy.util import attrgetter
+from sqlalchemy.util import attrgetter, Set
 
 
 __all__ = ['collection', 'collection_adapter',
@@ -504,11 +508,18 @@ class CollectionAdapter(object):
             return converter(obj)
 
         setting_type = sautil.duck_type_collection(obj)
+        receiving_type = sautil.duck_type_collection(self._data())
+
+        if obj is None or setting_type != receiving_type:
+            given = obj is None and 'None' or obj.__class__.__name__
+            if receiving_type is None:
+                wanted = self._data().__class__.__name__
+            else:
+                wanted = receiving_type.__name__
 
-        if obj is None or setting_type != self.attr.collection_interface:
             raise TypeError(
-                "Incompatible collection type: %s is not %s-like" %
-                (type(obj).__name__, self.attr.collection_interface.__name__))
+                "Incompatible collection type: %s is not %s-like" % (
+                given, wanted))
 
         # If the object is an adapted collection, return the (iterable) adapter.
         if getattr(obj, '_sa_adapter', None) is not None:
@@ -587,13 +598,13 @@ class CollectionAdapter(object):
 
     def fire_pre_remove_event(self, initiator=None):
         """Notify that an entity is about to be removed from the collection.
-        
-        Only called if the entity cannot be removed after calling 
+
+        Only called if the entity cannot be removed after calling
         fire_remove_event().
         """
-        
+
         self.attr.fire_pre_remove_event(self.owner_state, initiator=initiator)
-        
+
     def __getstate__(self):
         return { 'key': self.attr.key,
                  'owner_state': self.owner_state,
@@ -605,6 +616,43 @@ class CollectionAdapter(object):
         self._data = weakref.ref(d['data'])
 
 
+def bulk_replace(values, existing_adapter, new_adapter):
+    """Load a new collection, firing events based on prior like membership.
+
+    Appends instances in ``values`` onto the ``new_adapter``. Events will be
+    fired for any instance not present in the ``existing_adapter``.  Any
+    instances in ``existing_adapter`` not present in ``values`` will have
+    remove events fired upon them.
+
+    values
+      An iterable of collection member instances
+
+    existing_adapter
+      A CollectionAdapter of instances to be replaced
+
+    new_adapter
+      An empty CollectionAdapter to load with ``values``
+
+
+    """
+    if not isinstance(values, list):
+        values = list(values)
+
+    idset = sautil.IdentitySet
+    constants = idset(existing_adapter or ()).intersection(values or ())
+    additions = idset(values or ()).difference(constants)
+    removals  = idset(existing_adapter or ()).difference(constants)
+
+    for member in values or ():
+        if member in additions:
+            new_adapter.append_with_event(member)
+        elif member in constants:
+            new_adapter.append_without_event(member)
+
+    if existing_adapter:
+        for member in removals:
+            existing_adapter.remove_with_event(member)
+
 __instrumentation_mutex = sautil.threading.Lock()
 def _prepare_instrumentation(factory):
     """Prepare a callable for future use as a collection class factory.
@@ -706,7 +754,7 @@ def _instrument_class(cls):
     methods = roles.pop('methods', {})
 
     for name in dir(cls):
-        method = getattr(cls, name)
+        method = getattr(cls, name, None)
         if not callable(method):
             continue
 
@@ -853,7 +901,7 @@ def __before_delete(collection, _sa_initiator=None):
     executor = getattr(collection, '_sa_adapter', None)
     if executor:
         getattr(executor, 'fire_pre_remove_event')(_sa_initiator)
-    
+
 def _list_decorators():
     """Hand-turned instrumentation wrappers that can decorate any list-like
     class."""
@@ -877,8 +925,8 @@ def _list_decorators():
 
     def remove(fn):
         def remove(self, value, _sa_initiator=None):
-            # testlib.pragma exempt:__eq__
             __before_delete(self, _sa_initiator)
+            # testlib.pragma exempt:__eq__
             fn(self, value)
             __del(self, value, _sa_initiator)
         _tidy(remove)
@@ -1004,7 +1052,7 @@ def _dict_decorators():
         setattr(fn, '_sa_instrumented', True)
         fn.__doc__ = getattr(getattr(dict, fn.__name__), '__doc__')
 
-    Unspecified=object()
+    Unspecified=sautil.symbol('Unspecified')
 
     def __setitem__(fn):
         def __setitem__(self, key, value, _sa_initiator=None):
@@ -1092,33 +1140,58 @@ def _dict_decorators():
     l.pop('Unspecified')
     return l
 
+
+try:
+    _set_binop_bases = (set, frozenset, sets.BaseSet)
+except NameError:
+    _set_binop_bases = (sets.BaseSet,)
+
+def _set_binops_check_strict(self, obj):
+    """Allow only set, frozenset and self.__class__-derived objects in binops."""
+    return isinstance(obj, _set_binop_bases + (self.__class__,))
+
+def _set_binops_check_loose(self, obj):
+    """Allow anything set-like to participate in set binops."""
+    return (isinstance(obj, _set_binop_bases + (self.__class__,)) or
+            sautil.duck_type_collection(obj) == sautil.Set)
+
+
 def _set_decorators():
     """Hand-turned instrumentation wrappers that can decorate any set-like
     sequence class."""
 
     def _tidy(fn):
         setattr(fn, '_sa_instrumented', True)
-        fn.__doc__ = getattr(getattr(set, fn.__name__), '__doc__')
+        fn.__doc__ = getattr(getattr(Set, fn.__name__), '__doc__')
 
-    Unspecified=object()
+    Unspecified=sautil.symbol('Unspecified')
 
     def add(fn):
         def add(self, value, _sa_initiator=None):
-            __set(self, value, _sa_initiator)
+            if value not in self:
+                __set(self, value, _sa_initiator)
             # testlib.pragma exempt:__hash__
             fn(self, value)
         _tidy(add)
         return add
 
-    def discard(fn):
-        def discard(self, value, _sa_initiator=None):
-            # testlib.pragma exempt:__hash__
-            if value in self:
-                __del(self, value, _sa_initiator)
-            # testlib.pragma exempt:__hash__
-            fn(self, value)
-        _tidy(discard)
-        return discard
+    if sys.version_info < (2, 4):
+        def discard(fn):
+            def discard(self, value, _sa_initiator=None):
+                if value in self:
+                    self.remove(value, _sa_initiator)
+            _tidy(discard)
+            return discard
+    else:
+        def discard(fn):
+            def discard(self, value, _sa_initiator=None):
+                # testlib.pragma exempt:__hash__
+                if value in self:
+                    __del(self, value, _sa_initiator)
+                    # testlib.pragma exempt:__hash__
+                fn(self, value)
+            _tidy(discard)
+            return discard
 
     def remove(fn):
         def remove(self, value, _sa_initiator=None):
@@ -1149,18 +1222,16 @@ def _set_decorators():
     def update(fn):
         def update(self, value):
             for item in value:
-                if item not in self:
-                    self.add(item)
+                self.add(item)
         _tidy(update)
         return update
 
     def __ior__(fn):
         def __ior__(self, value):
-            if sautil.duck_type_collection(value) is not set:
+            if not _set_binops_check_strict(self, value):
                 return NotImplemented
             for item in value:
-                if item not in self:
-                    self.add(item)
+                self.add(item)
             return self
         _tidy(__ior__)
         return __ior__
@@ -1174,7 +1245,7 @@ def _set_decorators():
 
     def __isub__(fn):
         def __isub__(self, value):
-            if sautil.duck_type_collection(value) is not set:
+            if not _set_binops_check_strict(self, value):
                 return NotImplemented
             for item in value:
                 self.discard(item)
@@ -1184,7 +1255,7 @@ def _set_decorators():
 
     def intersection_update(fn):
         def intersection_update(self, other):
-            want, have = self.intersection(other), sautil.Set(self)
+            want, have = self.intersection(other), Set(self)
             remove, add = have - want, want - have
 
             for item in remove:
@@ -1196,9 +1267,9 @@ def _set_decorators():
 
     def __iand__(fn):
         def __iand__(self, other):
-            if sautil.duck_type_collection(other) is not set:
+            if not _set_binops_check_strict(self, other):
                 return NotImplemented
-            want, have = self.intersection(other), sautil.Set(self)
+            want, have = self.intersection(other), Set(self)
             remove, add = have - want, want - have
 
             for item in remove:
@@ -1211,7 +1282,7 @@ def _set_decorators():
 
     def symmetric_difference_update(fn):
         def symmetric_difference_update(self, other):
-            want, have = self.symmetric_difference(other), sautil.Set(self)
+            want, have = self.symmetric_difference(other), Set(self)
             remove, add = have - want, want - have
 
             for item in remove:
@@ -1223,9 +1294,9 @@ def _set_decorators():
 
     def __ixor__(fn):
         def __ixor__(self, other):
-            if sautil.duck_type_collection(other) is not set:
+            if not _set_binops_check_strict(self, other):
                 return NotImplemented
-            want, have = self.symmetric_difference(other), sautil.Set(self)
+            want, have = self.symmetric_difference(other), Set(self)
             remove, add = have - want, want - have
 
             for item in remove:
@@ -1250,7 +1321,7 @@ class InstrumentedList(list):
        'remover': 'remove',
        'iterator': '__iter__', }
 
-class InstrumentedSet(sautil.Set):
+class InstrumentedSet(Set):
     """An instrumented version of the built-in set (or Set)."""
 
     __instrumentation__ = {
@@ -1266,7 +1337,7 @@ class InstrumentedDict(dict):
 
 __canned_instrumentation = {
     list: InstrumentedList,
-    sautil.Set: InstrumentedSet,
+    Set: InstrumentedSet,
     dict: InstrumentedDict,
     }
 
@@ -1275,10 +1346,10 @@ __interfaces = {
             'remover':  'remove',
             'iterator': '__iter__',
             '_decorators': _list_decorators(), },
-    sautil.Set: { 'appender': 'add',
-                  'remover': 'remove',
-                  'iterator': '__iter__',
-                  '_decorators': _set_decorators(), },
+    Set: { 'appender': 'add',
+           'remover': 'remove',
+           'iterator': '__iter__',
+           '_decorators': _set_decorators(), },
     # decorators are required for dicts and object collections.
     dict: { 'iterator': 'itervalues',
             '_decorators': _dict_decorators(), },
@@ -1324,6 +1395,7 @@ class MappedCollection(dict):
 
         key = self.keyfunc(value)
         # Let self[key] raise if key is not in this collection
+        # testlib.pragma exempt:__ne__
         if self[key] != value:
             raise exceptions.InvalidRequestError(
                 "Can not remove '%s': collection holds '%s' for key '%s'. "
