@@ -10,10 +10,10 @@
  dependencies at flush time.
 """
 
-from sqlalchemy.orm import sync, attributes
-from sqlalchemy.orm.sync import ONETOMANY,MANYTOONE,MANYTOMANY
+from sqlalchemy.orm import sync
 from sqlalchemy import sql, util, exceptions
-from sqlalchemy.orm import session as sessionlib
+from sqlalchemy.orm.interfaces import ONETOMANY, MANYTOONE, MANYTOMANY
+
 
 def create_dependency_processor(prop):
     types = {
@@ -28,13 +28,12 @@ def create_dependency_processor(prop):
 
 class DependencyProcessor(object):
     no_dependencies = False
-    
+
     def __init__(self, prop):
         self.prop = prop
         self.cascade = prop.cascade
         self.mapper = prop.mapper
         self.parent = prop.parent
-        self.association = prop.association
         self.secondary = prop.secondary
         self.direction = prop.direction
         self.is_backref = prop.is_backref
@@ -44,8 +43,8 @@ class DependencyProcessor(object):
         self.passive_updates = prop.passive_updates
         self.enable_typechecks = prop.enable_typechecks
         self.key = prop.key
-
-        self._compile_synchronizers()
+        if not self.prop.synchronize_pairs:
+            raise exceptions.ArgumentError("Can't build a DependencyProcessor for relation %s.  No target attributes to populate between parent and child are present" % self.prop)
 
     def _get_instrumented_attribute(self):
         """Return the ``InstrumentedAttribute`` handled by this
@@ -55,12 +54,12 @@ class DependencyProcessor(object):
         return getattr(self.parent.class_, self.key)
 
     def hasparent(self, state):
-        """return True if the given object instance has a parent, 
+        """return True if the given object instance has a parent,
         according to the ``InstrumentedAttribute`` handled by this ``DependencyProcessor``."""
-        
+
         # TODO: use correct API for this
         return self._get_instrumented_attribute().impl.hasparent(state)
-        
+
     def register_dependencies(self, uowcommit):
         """Tell a ``UOWTransaction`` what mappers are dependent on
         which, with regards to the two or three mappers handled by
@@ -109,11 +108,12 @@ class DependencyProcessor(object):
         raise NotImplementedError()
 
     def _verify_canload(self, state):
-        if not self.enable_typechecks:
-            return
-        if state is not None and not self.mapper._canload(state):
-            raise exceptions.FlushError("Attempting to flush an item of type %s on collection '%s', which is handled by mapper '%s' and does not load items of that type.  Did you mean to use a polymorphic mapper for this relationship ?  Set 'enable_typechecks=False' on the relation() to disable this exception.  Mismatched typeloading may cause bi-directional relationships (backrefs) to not function properly." % (state.class_, self.prop, self.mapper))
-        
+        if state is not None and not self.mapper._canload(state, allow_subtypes=not self.enable_typechecks):
+            if self.mapper._canload(state, allow_subtypes=True):
+                raise exceptions.FlushError("Attempting to flush an item of type %s on collection '%s', which is not the expected type %s.  Configure mapper '%s' to load this subtype polymorphically, or set enable_typechecks=False to allow subtypes.  Mismatched typeloading may cause bi-directional relationships (backrefs) to not function properly." % (state.class_, self.prop, self.mapper.class_, self.mapper))
+            else:
+                raise exceptions.FlushError("Attempting to flush an item of type %s on collection '%s', whose mapper does not inherit from that of %s." % (state.class_, self.prop, self.mapper.class_))
+            
     def _synchronize(self, state, child, associationrow, clearkeys, uowcommit):
         """Called during a flush to synchronize primary key identifier
         values between a parent/child object, as well as to an
@@ -122,20 +122,6 @@ class DependencyProcessor(object):
 
         raise NotImplementedError()
 
-    def _compile_synchronizers(self):
-        """Assemble a list of *synchronization rules*.
-                
-        These are fired to populate attributes from one side
-        of a relation to another.
-        """
-
-        self.syncrules = sync.ClauseSynchronizer(self.parent, self.mapper, self.direction)
-        if self.direction == sync.MANYTOMANY:
-            self.syncrules.compile(self.prop.primaryjoin, issecondary=False, foreign_keys=self.foreign_keys)
-            self.syncrules.compile(self.prop.secondaryjoin, issecondary=True, foreign_keys=self.foreign_keys)
-        else:
-            self.syncrules.compile(self.prop.primaryjoin, foreign_keys=self.foreign_keys)
-
 
     def _conditional_post_update(self, state, uowcommit, related):
         """Execute a post_update call.
@@ -154,12 +140,12 @@ class DependencyProcessor(object):
         if state is not None and self.post_update:
             for x in related:
                 if x is not None:
-                    uowcommit.register_object(state, postupdate=True, post_update_cols=self.syncrules.dest_columns())
+                    uowcommit.register_object(state, postupdate=True, post_update_cols=[r for l, r in self.prop.synchronize_pairs])
                     break
-    
+
     def _pks_changed(self, uowcommit, state):
-        return self.syncrules.source_changes(uowcommit, state)
-        
+        raise NotImplementedError()
+
     def __str__(self):
         return "%s(%s)" % (self.__class__.__name__, str(self.prop))
 
@@ -182,7 +168,7 @@ class OneToManyDP(DependencyProcessor):
             # the child objects have to have their foreign key to the parent set to NULL
             # this phase can be called safely for any cascade but is unnecessary if delete cascade
             # is on.
-            if (not self.cascade.delete or self.post_update) and not self.passive_deletes=='all':
+            if self.post_update or not self.passive_deletes=='all':
                 for state in deplist:
                     (added, unchanged, deleted) = uowcommit.get_attribute_history(state, self.key,passive=self.passive_deletes)
                     if unchanged or deleted:
@@ -190,10 +176,11 @@ class OneToManyDP(DependencyProcessor):
                             if child is not None and self.hasparent(child) is False:
                                 self._synchronize(state, child, None, True, uowcommit)
                                 self._conditional_post_update(child, uowcommit, [state])
-                        for child in unchanged:
-                            if child is not None:
-                                self._synchronize(state, child, None, True, uowcommit)
-                                self._conditional_post_update(child, uowcommit, [state])
+                        if self.post_update or not self.cascade.delete:
+                            for child in unchanged:
+                                if child is not None:
+                                    self._synchronize(state, child, None, True, uowcommit)
+                                    self._conditional_post_update(child, uowcommit, [state])
         else:
             for state in deplist:
                 (added, unchanged, deleted) = uowcommit.get_attribute_history(state, self.key, passive=True)
@@ -205,28 +192,33 @@ class OneToManyDP(DependencyProcessor):
                     for child in deleted:
                         if not self.cascade.delete_orphan and not self.hasparent(child):
                             self._synchronize(state, child, None, True, uowcommit)
-                            
+
                 if self._pks_changed(uowcommit, state):
                     if unchanged:
                         for child in unchanged:
                             self._synchronize(state, child, None, False, uowcommit)
-                
+
     def preprocess_dependencies(self, task, deplist, uowcommit, delete = False):
         #print self.mapper.mapped_table.name + " " + self.key + " " + repr(len(deplist)) + " preprocess_dep isdelete " + repr(delete) + " direction " + repr(self.direction)
 
         if delete:
             # head object is being deleted, and we manage its list of child objects
             # the child objects have to have their foreign key to the parent set to NULL
-            if not self.post_update and not self.cascade.delete and not self.passive_deletes=='all':
+            if not self.post_update:
+                should_null_fks = not self.cascade.delete and not self.passive_deletes=='all'
                 for state in deplist:
                     (added, unchanged, deleted) = uowcommit.get_attribute_history(state, self.key,passive=self.passive_deletes)
                     if unchanged or deleted:
                         for child in deleted:
                             if child is not None and self.hasparent(child) is False:
-                                uowcommit.register_object(child)
-                        for child in unchanged:
-                            if child is not None:
-                                uowcommit.register_object(child)
+                                if self.cascade.delete_orphan:
+                                    uowcommit.register_object(child, isdelete=True)
+                                else:
+                                    uowcommit.register_object(child)
+                        if should_null_fks:
+                            for child in unchanged:
+                                if child is not None:
+                                    uowcommit.register_object(child)
         else:
             for state in deplist:
                 (added, unchanged, deleted) = uowcommit.get_attribute_history(state, self.key,passive=True)
@@ -247,61 +239,69 @@ class OneToManyDP(DependencyProcessor):
                     if unchanged:
                         for child in unchanged:
                             uowcommit.register_object(child)
-                    
+
     def _synchronize(self, state, child, associationrow, clearkeys, uowcommit):
-        if child is not None:
-            child = getattr(child, '_state', child)
         source = state
         dest = child
         if dest is None or (not self.post_update and uowcommit.is_deleted(dest)):
             return
         self._verify_canload(child)
-        self.syncrules.execute(source, dest, source, child, clearkeys)
+        if clearkeys:
+            sync.clear(dest, self.mapper, self.prop.synchronize_pairs)
+        else:
+            sync.populate(source, self.parent, dest, self.mapper, self.prop.synchronize_pairs)
+
+    def _pks_changed(self, uowcommit, state):
+        return sync.source_changes(uowcommit, state, self.parent, self.prop.synchronize_pairs)
 
 class DetectKeySwitch(DependencyProcessor):
     """a special DP that works for many-to-one relations, fires off for
     child items who have changed their referenced key."""
 
     no_dependencies = True
-    
+
     def register_dependencies(self, uowcommit):
         uowcommit.register_processor(self.parent, self, self.mapper)
-        
+
     def preprocess_dependencies(self, task, deplist, uowcommit, delete=False):
         # for non-passive updates, register in the preprocess stage
         # so that mapper save_obj() gets a hold of changes
         if not delete and not self.passive_updates:
             self._process_key_switches(deplist, uowcommit)
-        
+
     def process_dependencies(self, task, deplist, uowcommit, delete=False):
         # for passive updates, register objects in the process stage
-        # so that we avoid ManyToOneDP's registering the object without 
+        # so that we avoid ManyToOneDP's registering the object without
         # the listonly flag in its own preprocess stage (results in UPDATE)
         # statements being emitted
         if not delete and self.passive_updates:
             self._process_key_switches(deplist, uowcommit)
-    
-    def _process_key_switches(self, deplist, uowcommit):    
-        switchers = util.Set(s for s in deplist if self._pks_changed(uowcommit, s))
+
+    def _process_key_switches(self, deplist, uowcommit):
+        switchers = util.Set([s for s in deplist if self._pks_changed(uowcommit, s)])
         if switchers:
-            # yes, we're doing a linear search right now through the UOW.  only 
+            # yes, we're doing a linear search right now through the UOW.  only
             # takes effect when primary key values have actually changed.
             # a possible optimization might be to enhance the "hasparents" capability of
             # attributes to actually store all parent references, but this introduces
             # more complicated attribute accounting.
-            for s in [elem for elem in uowcommit.session.identity_map.all_states() 
-                if issubclass(elem.class_, self.parent.class_) and 
-                    self.key in elem.dict and 
+            for s in [elem for elem in uowcommit.session.identity_map.all_states()
+                if issubclass(elem.class_, self.parent.class_) and
+                    self.key in elem.dict and
                     elem.dict[self.key]._state in switchers
                 ]:
                 uowcommit.register_object(s, listonly=self.passive_updates)
-                self.syncrules.execute(s.dict[self.key]._state, s, None, None, False)
-                        
+                sync.populate(s.dict[self.key]._state, self.mapper, s, self.parent, self.prop.synchronize_pairs)
+                #self.syncrules.execute(s.dict[self.key]._state, s, None, None, False)
+
+    def _pks_changed(self, uowcommit, state):
+        return sync.source_changes(uowcommit, state, self.mapper, self.prop.synchronize_pairs)
+
 class ManyToOneDP(DependencyProcessor):
     def __init__(self, prop):
         DependencyProcessor.__init__(self, prop)
         self.mapper._dependency_processors.append(DetectKeySwitch(prop))
-        
+
     def register_dependencies(self, uowcommit):
         if self.post_update:
             if not self.is_backref:
@@ -312,7 +312,7 @@ class ManyToOneDP(DependencyProcessor):
         else:
             uowcommit.register_dependency(self.mapper, self.parent)
             uowcommit.register_processor(self.mapper, self, self.parent)
-            
+
 
     def process_dependencies(self, task, deplist, uowcommit, delete = False):
         #print self.mapper.mapped_table.name + " " + self.key + " " + repr(len(deplist)) + " process_dep isdelete " + repr(delete) + " direction " + repr(self.direction)
@@ -338,15 +338,19 @@ class ManyToOneDP(DependencyProcessor):
         if self.post_update:
             return
         if delete:
-            if self.cascade.delete:
+            if self.cascade.delete or self.cascade.delete_orphan:
                 for state in deplist:
                     (added, unchanged, deleted) = uowcommit.get_attribute_history(state, self.key,passive=self.passive_deletes)
-                    if deleted or unchanged:
-                        for child in deleted + unchanged:
-                            if child is not None and self.hasparent(child) is False:
-                                uowcommit.register_object(child, isdelete=True)
-                                for c, m in self.mapper.cascade_iterator('delete', child):
-                                    uowcommit.register_object(c._state, isdelete=True)
+                    if self.cascade.delete_orphan:
+                        todelete = added + unchanged + deleted
+                    else:
+                        todelete = added + unchanged
+                    for child in todelete:
+                        if child is None:
+                            continue
+                        uowcommit.register_object(child, isdelete=True)
+                        for c, m in self.mapper.cascade_iterator('delete', child):
+                            uowcommit.register_object(c._state, isdelete=True)
         else:
             for state in deplist:
                 uowcommit.register_object(state)
@@ -361,12 +365,14 @@ class ManyToOneDP(DependencyProcessor):
 
 
     def _synchronize(self, state, child, associationrow, clearkeys, uowcommit):
-        source = child
-        dest = state
-        if dest is None or (not self.post_update and uowcommit.is_deleted(dest)):
+        if state is None or (not self.post_update and uowcommit.is_deleted(state)):
             return
-        self._verify_canload(child)
-        self.syncrules.execute(source, dest, dest, child, clearkeys)
+
+        if clearkeys or child is None:
+            sync.clear(state, self.parent, self.prop.synchronize_pairs)
+        else:
+            self._verify_canload(child)
+            sync.populate(child, self.mapper, state, self.parent, self.prop.synchronize_pairs)
 
 class ManyToManyDP(DependencyProcessor):
     def register_dependencies(self, uowcommit):
@@ -387,12 +393,12 @@ class ManyToManyDP(DependencyProcessor):
         secondary_delete = []
         secondary_insert = []
         secondary_update = []
-        
-        if hasattr(self.prop, 'reverse_property'):
-            reverse_dep = getattr(self.prop.reverse_property, '_dependency_processor', None)
+
+        if self.prop._reverse_property:
+            reverse_dep = getattr(self.prop._reverse_property, '_dependency_processor', None)
         else:
             reverse_dep = None
-            
+
         if delete:
             for state in deplist:
                 (added, unchanged, deleted) = uowcommit.get_attribute_history(state, self.key,passive=self.passive_deletes)
@@ -422,27 +428,30 @@ class ManyToManyDP(DependencyProcessor):
                         self._synchronize(state, child, associationrow, False, uowcommit)
                         uowcommit.attributes[(self, "manytomany", state, child)] = True
                         secondary_delete.append(associationrow)
-                        
+
                 if not self.passive_updates and unchanged and self._pks_changed(uowcommit, state):
                     for child in unchanged:
                         associationrow = {}
-                        self.syncrules.update(associationrow, state, child, "old_")
+                        sync.update(state, self.parent, associationrow, "old_", self.prop.synchronize_pairs)
+                        sync.update(child, self.mapper, associationrow, "old_", self.prop.secondary_synchronize_pairs)
+
+                        #self.syncrules.update(associationrow, state, child, "old_")
                         secondary_update.append(associationrow)
-                    
+
         if secondary_delete:
             secondary_delete.sort()
             # TODO: precompile the delete/insert queries?
             statement = self.secondary.delete(sql.and_(*[c == sql.bindparam(c.key, type_=c.type) for c in self.secondary.c if c.key in associationrow]))
             result = connection.execute(statement, secondary_delete)
             if result.supports_sane_multi_rowcount() and result.rowcount != len(secondary_delete):
-                raise exceptions.ConcurrentModificationError("Deleted rowcount %d does not match number of objects deleted %d" % (result.rowcount, len(secondary_delete)))
-        
+                raise exceptions.ConcurrentModificationError("Deleted rowcount %d does not match number of secondary table rows deleted from table '%s': %d" % (result.rowcount, self.secondary.description, len(secondary_delete)))
+
         if secondary_update:
             statement = self.secondary.update(sql.and_(*[c == sql.bindparam("old_" + c.key, type_=c.type) for c in self.secondary.c if c.key in associationrow]))
             result = connection.execute(statement, secondary_update)
             if result.supports_sane_multi_rowcount() and result.rowcount != len(secondary_update):
-                raise exceptions.ConcurrentModificationError("Updated rowcount %d does not match number of objects updated %d" % (result.rowcount, len(secondary_update)))
-            
+                raise exceptions.ConcurrentModificationError("Updated rowcount %d does not match number of secondary table rows updated from table '%s': %d" % (result.rowcount, self.secondary.description, len(secondary_update)))
+
         if secondary_insert:
             statement = self.secondary.insert()
             connection.execute(statement, secondary_insert)
@@ -463,7 +472,12 @@ class ManyToManyDP(DependencyProcessor):
         if associationrow is None:
             return
         self._verify_canload(child)
-        self.syncrules.execute(None, associationrow, state, child, clearkeys)
+        
+        sync.populate_dict(state, self.parent, associationrow, self.prop.synchronize_pairs)
+        sync.populate_dict(child, self.mapper, associationrow, self.prop.secondary_synchronize_pairs)
+
+    def _pks_changed(self, uowcommit, state):
+        return sync.source_changes(uowcommit, state, self.parent, self.prop.synchronize_pairs)
 
 class AssociationDP(OneToManyDP):
     def __init__(self, *args, **kwargs):
@@ -481,7 +495,7 @@ class MapperStub(object):
     """
 
     __metaclass__ = util.ArgSingleton
-    
+
     def __init__(self, parent, mapper, key):
         self.mapper = mapper
         self.base_mapper = self
@@ -490,7 +504,7 @@ class MapperStub(object):
 
     def polymorphic_iterator(self):
         return iter([self])
-        
+
     def _register_dependencies(self, uowcommit):
         pass
 
@@ -502,4 +516,3 @@ class MapperStub(object):
 
     def primary_mapper(self):
         return self
-
