@@ -1,5 +1,5 @@
 from sqlalchemy.util import ScopedRegistry, to_list, get_cls_kwargs
-from sqlalchemy.orm import MapperExtension, EXT_CONTINUE, object_session
+from sqlalchemy.orm import MapperExtension, EXT_CONTINUE, object_session, class_mapper
 from sqlalchemy.orm.session import Session
 from sqlalchemy import exceptions
 import types
@@ -69,11 +69,35 @@ class ScopedSession(object):
         
         self.session_factory.configure(**kwargs)
 
+    def query_property(self):
+        """return a class property which produces a `Query` object against the
+        class when called.
+        
+        e.g.::
+            Session = scoped_session(sessionmaker())
+            
+            class MyClass(object):
+                query = Session.query_property()
+                
+            # after mappers are defined
+            result = MyClass.query.filter(MyClass.name=='foo').all()
+        
+        """
+        
+        class query(object):
+            def __get__(s, instance, owner):
+                mapper = class_mapper(owner, raiseerror=False)
+                if mapper:
+                    return self.registry().query(mapper)
+                else:
+                    return None
+        return query()
+        
 def instrument(name):
     def do(self, *args, **kwargs):
         return getattr(self.registry(), name)(*args, **kwargs)
     return do
-for meth in ('get', 'load', 'close', 'save', 'commit', 'update', 'save_or_update', 'flush', 'query', 'delete', 'merge', 'clear', 'refresh', 'expire', 'expunge', 'rollback', 'begin', 'begin_nested', 'connection', 'execute', 'scalar', 'get_bind', 'is_modified', '__contains__', '__iter__'):
+for meth in ('add', 'add_all', 'get', 'load', 'close', 'save', 'commit', 'update', 'save_or_update', 'flush', 'query', 'delete', 'merge', 'clear', 'refresh', 'expire', 'expunge', 'rollback', 'begin', 'begin_nested', 'connection', 'execute', 'scalar', 'get_bind', 'is_modified', '__contains__', '__iter__'):
     setattr(ScopedSession, meth, instrument(meth))
 
 def makeprop(name):
