@@ -192,7 +192,7 @@ class LoadDeferredColumns(object):
 
         session = sessionlib.object_session(self.instance)
         if session is None:
-            raise exceptions.InvalidRequestError("Parent instance %s is not bound to a Session; deferred load operation of attribute '%s' cannot proceed" % (self.instance.__class__, self.key))
+            raise exceptions.UnboundExecutionError("Parent instance %s is not bound to a Session; deferred load operation of attribute '%s' cannot proceed" % (self.instance.__class__, self.key))
 
         query = session.query(localparent)
         if not self.optimizing_statement:
@@ -223,7 +223,7 @@ class UndeferGroupOption(MapperOption):
 class AbstractRelationLoader(LoaderStrategy):
     def init(self):
         super(AbstractRelationLoader, self).init()
-        for attr in ['primaryjoin', 'secondaryjoin', 'secondary', 'foreign_keys', 'mapper', 'select_mapper', 'target', 'select_table', 'loads_polymorphic', 'uselist', 'cascade', 'attributeext', 'order_by', 'remote_side', 'polymorphic_primaryjoin', 'polymorphic_secondaryjoin', 'direction']:
+        for attr in ['primaryjoin', 'secondaryjoin', 'secondary', 'foreign_keys', 'mapper', 'target', 'table', 'uselist', 'cascade', 'attributeext', 'order_by', 'remote_side', 'direction']:
             setattr(self, attr, getattr(self.parent_property, attr))
         self._should_log_debug = logging.is_debug_enabled(self.logger)
         
@@ -237,16 +237,6 @@ class AbstractRelationLoader(LoaderStrategy):
         self.logger.info("register managed %s attribute %s on class %s" % ((self.uselist and "list-holding" or "scalar"), self.key, self.parent.class_.__name__))
         sessionlib.register_attribute(class_, self.key, uselist=self.uselist, useobject=True, extension=self.attributeext, cascade=self.cascade,  trackparent=True, typecallable=self.parent_property.collection_class, callable_=callable_, comparator=self.parent_property.comparator, **kwargs)
 
-class DynaLoader(AbstractRelationLoader):
-    def init_class_attribute(self):
-        self.is_class_level = True
-        self._register_attribute(self.parent.class_, dynamic=True, target_mapper=self.parent_property.mapper)
-
-    def create_row_processor(self, selectcontext, mapper, row):
-        return (None, None, None)
-
-DynaLoader.logger = logging.class_logger(DynaLoader)
-        
 class NoLoader(AbstractRelationLoader):
     def init_class_attribute(self):
         self.is_class_level = True
@@ -265,14 +255,14 @@ NoLoader.logger = logging.class_logger(NoLoader)
 class LazyLoader(AbstractRelationLoader):
     def init(self):
         super(LazyLoader, self).init()
-        (self.lazywhere, self.lazybinds, self.equated_columns) = self._create_lazy_clause(self)
+        (self.__lazywhere, self.__bind_to_col, self._equated_columns) = self.__create_lazy_clause(self.parent_property)
         
-        self.logger.info(str(self.parent_property) + " lazy loading clause " + str(self.lazywhere))
+        self.logger.info(str(self.parent_property) + " lazy loading clause " + str(self.__lazywhere))
 
         # determine if our "lazywhere" clause is the same as the mapper's
         # get() clause.  then we can just use mapper.get()
         #from sqlalchemy.orm import query
-        self.use_get = not self.uselist and self.mapper._get_clause[0].compare(self.lazywhere)
+        self.use_get = not self.uselist and self.mapper._get_clause[0].compare(self.__lazywhere)
         if self.use_get:
             self.logger.info(str(self.parent_property) + " will use query.get() to optimize instance loads")
 
@@ -280,29 +270,33 @@ class LazyLoader(AbstractRelationLoader):
         self.is_class_level = True
         self._register_attribute(self.parent.class_, callable_=self.class_level_loader)
 
-    def lazy_clause(self, instance, reverse_direction=False):
+    def lazy_clause(self, instance, reverse_direction=False, alias_secondary=False):
         if instance is None:
             return self._lazy_none_clause(reverse_direction)
             
         if not reverse_direction:
-            (criterion, lazybinds, rev) = (self.lazywhere, self.lazybinds, self.equated_columns)
+            (criterion, bind_to_col, rev) = (self.__lazywhere, self.__bind_to_col, self._equated_columns)
         else:
-            (criterion, lazybinds, rev) = LazyLoader._create_lazy_clause(self.parent_property, reverse_direction=reverse_direction)
-        bind_to_col = dict([(lazybinds[col].key, col) for col in lazybinds])
+            (criterion, bind_to_col, rev) = LazyLoader.__create_lazy_clause(self.parent_property, reverse_direction=reverse_direction)
 
         def visit_bindparam(bindparam):
             mapper = reverse_direction and self.parent_property.mapper or self.parent_property.parent
             if bindparam.key in bind_to_col:
                 # use the "committed" (database) version to get query column values
-                bindparam.value = mapper._get_committed_attr_by_column(instance, bind_to_col[bindparam.key])
+                # also its a deferred value; so that when used by Query, the committed value is used
+                # after an autoflush occurs
+                bindparam.value = lambda: mapper._get_committed_attr_by_column(instance, bind_to_col[bindparam.key])
+                
+        if self.secondary and alias_secondary:
+            criterion = sql_util.ClauseAdapter(self.secondary.alias()).traverse(criterion)
+            
         return visitors.traverse(criterion, clone=True, visit_bindparam=visit_bindparam)
     
     def _lazy_none_clause(self, reverse_direction=False):
         if not reverse_direction:
-            (criterion, lazybinds, rev) = (self.lazywhere, self.lazybinds, self.equated_columns)
+            (criterion, bind_to_col, rev) = (self.__lazywhere, self.__bind_to_col, self._equated_columns)
         else:
-            (criterion, lazybinds, rev) = LazyLoader._create_lazy_clause(self.parent_property, reverse_direction=reverse_direction)
-        bind_to_col = dict([(lazybinds[col].key, col) for col in lazybinds])
+            (criterion, bind_to_col, rev) = LazyLoader.__create_lazy_clause(self.parent_property, reverse_direction=reverse_direction)
 
         def visit_binary(binary):
             mapper = reverse_direction and self.parent_property.mapper or self.parent_property.parent
@@ -359,46 +353,47 @@ class LazyLoader(AbstractRelationLoader):
                     instance._state.reset(self.key)
             return (new_execute, None, None)
 
-    def _create_lazy_clause(cls, prop, reverse_direction=False):
-        (primaryjoin, secondaryjoin, remote_side) = (prop.polymorphic_primaryjoin, prop.polymorphic_secondaryjoin, prop.remote_side)
-        
+    def __create_lazy_clause(cls, prop, reverse_direction=False):
         binds = {}
+        lookup = {}
         equated_columns = {}
 
-        def should_bind(targetcol, othercol):
-            if reverse_direction and not secondaryjoin:
-                return targetcol in remote_side
-            else:
-                return othercol in remote_side
-
-        def visit_binary(binary):
-            if not isinstance(binary.left, sql.ColumnElement) or not isinstance(binary.right, sql.ColumnElement):
-                return
-            leftcol = binary.left
-            rightcol = binary.right
-
-            equated_columns[rightcol] = leftcol
-            equated_columns[leftcol] = rightcol
-
-            if should_bind(leftcol, rightcol):
-                binary.left = binds[leftcol] = sql.bindparam(None, None, type_=binary.right.type)
-
-            # the "left is not right" compare is to handle part of a join clause that is "table.c.col1==table.c.col1",
-            # which can happen in rare cases (test/orm/relationships.py RelationTest2)
-            if leftcol is not rightcol and should_bind(rightcol, leftcol):
-                binary.right = binds[rightcol] = sql.bindparam(None, None, type_=binary.left.type)
-
-        lazywhere = primaryjoin
+        if reverse_direction and not prop.secondaryjoin:
+            for l, r in prop.local_remote_pairs:
+                _list = lookup.setdefault(r, [])
+                _list.append((r, l))
+                equated_columns[l] = r
+        else:
+            for l, r in prop.local_remote_pairs:
+                _list = lookup.setdefault(l, [])
+                _list.append((l, r))
+                equated_columns[r] = l
+                
+        def col_to_bind(col):
+            if col in lookup:
+                for tobind, equated in lookup[col]:
+                    if equated in binds:
+                        return None
+                if col not in binds:
+                    binds[col] = sql.bindparam(None, None, type_=col.type)
+                return binds[col]
+            return None
+                    
+        lazywhere = prop.primaryjoin
         
-        if not secondaryjoin or not reverse_direction:
-            lazywhere = visitors.traverse(lazywhere, clone=True, visit_binary=visit_binary)
+        if not prop.secondaryjoin or not reverse_direction:
+            lazywhere = visitors.traverse(lazywhere, before_clone=col_to_bind, clone=True) 
         
-        if secondaryjoin is not None:
+        if prop.secondaryjoin is not None:
+            secondaryjoin = prop.secondaryjoin
             if reverse_direction:
-                secondaryjoin = visitors.traverse(secondaryjoin, clone=True, visit_binary=visit_binary)
+                secondaryjoin = visitors.traverse(secondaryjoin, before_clone=col_to_bind, clone=True)
             lazywhere = sql.and_(lazywhere, secondaryjoin)
-        return (lazywhere, binds, equated_columns)
-    _create_lazy_clause = classmethod(_create_lazy_clause)
+    
+        bind_to_col = dict([(binds[col].key, col) for col in binds])
+        
+        return (lazywhere, bind_to_col, equated_columns)
+    __create_lazy_clause = classmethod(__create_lazy_clause)
     
 LazyLoader.logger = logging.class_logger(LazyLoader)
 
@@ -438,7 +433,7 @@ class LoadLazyAttribute(object):
             try:
                 session = instance_mapper.get_session()
             except exceptions.InvalidRequestError:
-                raise exceptions.InvalidRequestError("Parent instance %s is not bound to a Session, and no contextual session is established; lazy load operation of attribute '%s' cannot proceed" % (instance.__class__, self.key))
+                raise exceptions.UnboundExecutionError("Parent instance %s is not bound to a Session, and no contextual session is established; lazy load operation of attribute '%s' cannot proceed" % (instance.__class__, self.key))
 
         q = session.query(prop.mapper).autoflush(False)
         if self.path:
@@ -449,8 +444,8 @@ class LoadLazyAttribute(object):
         if strategy.use_get:
             ident = []
             allnulls = True
-            for primary_key in prop.select_mapper.primary_key: 
-                val = instance_mapper._get_committed_attr_by_column(instance, strategy.equated_columns[primary_key])
+            for primary_key in prop.mapper.primary_key: 
+                val = instance_mapper._get_committed_attr_by_column(instance, strategy._equated_columns[primary_key])
                 allnulls = allnulls and val is None
                 ident.append(val)
             if allnulls:
@@ -511,6 +506,28 @@ class EagerLoader(AbstractRelationLoader):
                 if self.mapper.base_mapper in path:
                     return
 
+        if ("eager_row_processor", path) in context.attributes:
+            # if user defined eager_row_processor, that's contains_eager().
+            # don't render LEFT OUTER JOIN, generate an AliasedClauses from 
+            # the decorator (this is a hack here, cleaned up in 0.5)
+            cl = context.attributes[("eager_row_processor", path)]
+            if cl:
+                row = cl(None)
+                class ActsLikeAliasedClauses(object):
+                    def aliased_column(self, col):
+                        return row.map[col]
+                clauses = ActsLikeAliasedClauses()
+            else:
+                clauses = None
+        else:
+            clauses = self.__create_eager_join(context, path, parentclauses, parentmapper, **kwargs)
+            if not clauses:
+                return
+
+        for value in self.mapper._iterate_polymorphic_properties():
+            context.exec_with_path(self.mapper, value.key, value.setup, context, parentclauses=clauses, parentmapper=self.mapper)
+
+    def __create_eager_join(self, context, path, parentclauses, parentmapper, **kwargs):
         if parentmapper is None:
             localparent = context.mapper
         else:
@@ -519,16 +536,13 @@ class EagerLoader(AbstractRelationLoader):
         if context.eager_joins:
             towrap = context.eager_joins
         else:
-            if isinstance(context.from_clause, sql.Join):
-                towrap = context.from_clause
-            else:
-                towrap = localparent.mapped_table
+            towrap = context.from_clause
         
         # create AliasedClauses object to build up the eager query.  this is cached after 1st creation.    
         try:
             clauses = self.clauses[path]
         except KeyError:
-            clauses = mapperutil.PropertyAliasedClauses(self.parent_property, self.parent_property.polymorphic_primaryjoin, self.parent_property.polymorphic_secondaryjoin, parentclauses)
+            clauses = mapperutil.PropertyAliasedClauses(self.parent_property, self.parent_property.primaryjoin, self.parent_property.secondaryjoin, parentclauses)
             self.clauses[path] = clauses
 
         # place the "row_decorator" from the AliasedClauses into the QueryContext, where it will
@@ -545,7 +559,6 @@ class EagerLoader(AbstractRelationLoader):
                 context.eager_order_by += clauses.secondary.default_order_by()
         else:
             context.eager_joins = towrap.outerjoin(clauses.alias, clauses.primaryjoin)
-
             # ensure all the cols on the parent side are actually in the
             # columns clause (i.e. are not deferred), so that aliasing applied by the Query propagates 
             # those columns outward.  This has the effect of "undefering" those columns.
@@ -559,8 +572,7 @@ class EagerLoader(AbstractRelationLoader):
         if clauses.order_by:
             context.eager_order_by += util.to_list(clauses.order_by)
         
-        for value in self.select_mapper.iterate_properties:
-            context.exec_with_path(self.select_mapper, value.key, value.setup, context, parentclauses=clauses, parentmapper=self.select_mapper)
+        return clauses
         
     def _create_row_decorator(self, selectcontext, row, path):
         """Create a *row decorating* function that will apply eager
@@ -584,7 +596,7 @@ class EagerLoader(AbstractRelationLoader):
         try:
             decorated_row = decorator(row)
             # check for identity key
-            identity_key = self.select_mapper.identity_key_from_row(decorated_row)
+            identity_key = self.mapper.identity_key_from_row(decorated_row)
             # and its good
             return decorator
         except KeyError, k:
@@ -596,6 +608,7 @@ class EagerLoader(AbstractRelationLoader):
     def create_row_processor(self, selectcontext, mapper, row):
 
         row_decorator = self._create_row_decorator(selectcontext, row, selectcontext.path)
+        pathstr = ','.join([str(x) for x in selectcontext.path])
         if row_decorator is not None:
             def execute(instance, row, isnew, **flags):
                 decorated_row = row_decorator(row)
@@ -608,11 +621,11 @@ class EagerLoader(AbstractRelationLoader):
                         # parent object, bypassing InstrumentedAttribute
                         # event handlers.
                         #
-                        instance.__dict__[self.key] = self.select_mapper._instance(selectcontext, decorated_row, None)
+                        instance.__dict__[self.key] = self.mapper._instance(selectcontext, decorated_row, None)
                     else:
                         # call _instance on the row, even though the object has been created,
                         # so that we further descend into properties
-                        self.select_mapper._instance(selectcontext, decorated_row, None)
+                        self.mapper._instance(selectcontext, decorated_row, None)
                 else:
                     if isnew or self.key not in instance._state.appenders:
                         # appender_key can be absent from selectcontext.attributes with isnew=False
@@ -630,8 +643,8 @@ class EagerLoader(AbstractRelationLoader):
                     result_list = instance._state.appenders[self.key]
                     if self._should_log_debug:
                         self.logger.debug("eagerload list instance on %s" % mapperutil.attribute_str(instance, self.key))
-
-                    self.select_mapper._instance(selectcontext, decorated_row, result_list)
+                    
+                    self.mapper._instance(selectcontext, decorated_row, result_list)
 
             if self._should_log_debug:
                 self.logger.debug("Returning eager instance loader for %s" % str(self))
@@ -680,22 +693,6 @@ class EagerLazyOption(StrategizedOption):
 
 EagerLazyOption.logger = logging.class_logger(EagerLazyOption)
 
-# TODO: enable FetchMode option.  currently 
-# this class does nothing.  will require Query
-# to swich between using its "polymorphic" selectable
-# and its regular selectable in order to make decisions
-# (therefore might require that FetchModeOperation is performed
-# only as the first operation on a Query.)
-class FetchModeOption(PropertyOption):
-    def __init__(self, key, type):
-        super(FetchModeOption, self).__init__(key)
-        if type not in ('join', 'select'):
-            raise exceptions.ArgumentError("Fetchmode must be one of 'join' or 'select'")
-        self.type = type
-        
-    def process_query_property(self, query, properties):
-        query.attributes[('fetchmode', properties[-1])] = self.type
-        
 class RowDecorateOption(PropertyOption):
     def __init__(self, key, decorator=None, alias=None):
         super(RowDecorateOption, self).__init__(key)
@@ -710,7 +707,7 @@ class RowDecorateOption(PropertyOption):
             if isinstance(self.alias, basestring):
                 self.alias = prop.target.alias(self.alias)
 
-            self.decorator = mapperutil.create_row_adapter(self.alias, prop.target)
+            self.decorator = mapperutil.create_row_adapter(self.alias)
         query._attributes[("eager_row_processor", paths[-1])] = self.decorator
 
 RowDecorateOption.logger = logging.class_logger(RowDecorateOption)
