@@ -20,7 +20,7 @@ import weakref, time
 
 from sqlalchemy import exceptions, logging
 from sqlalchemy import queue as Queue
-from sqlalchemy.util import thread, threading, pickle
+from sqlalchemy.util import thread, threading, pickle, as_interface
 
 proxies = {}
 
@@ -106,21 +106,28 @@ class Pool(object):
       newly opened connection. Defaults to -1.
 
     listeners
-      A list of ``PoolListener``-like objects that receive events when
-      DB-API connections are created, checked out and checked in to
-      the pool.
-    """
+      A list of ``PoolListener``-like objects or dictionaries of callables
+      that receive events when DB-API connections are created, checked out and
+      checked in to the pool.
 
+    reset_on_return
+      Defaults to True.  Reset the database state of connections returned to
+      the pool.  This is typically a ROLLBACK to release locks and transaction
+      resources.  Disable at your own peril.
+
+    """
     def __init__(self, creator, recycle=-1, echo=None, use_threadlocal=True,
-                 listeners=None):
+                 reset_on_return=True, listeners=None):
         self.logger = logging.instance_logger(self, echoflag=echo)
-        # the WeakValueDictionary works more nicely than a regular dict
-        # of weakrefs.  the latter can pile up dead reference objects which don't
-        # get cleaned out.  WVD adds from 1-6 method calls to a checkout operation.
+        # the WeakValueDictionary works more nicely than a regular dict of
+        # weakrefs.  the latter can pile up dead reference objects which don't
+        # get cleaned out.  WVD adds from 1-6 method calls to a checkout
+        # operation.
         self._threadconns = weakref.WeakValueDictionary()
         self._creator = creator
         self._recycle = recycle
         self._use_threadlocal = use_threadlocal
+        self._reset_on_return = reset_on_return
         self.echo = echo
         self.listeners = []
         self._on_connect = []
@@ -182,7 +189,16 @@ class Pool(object):
         raise NotImplementedError()
 
     def add_listener(self, listener):
-        """Add a ``PoolListener``-like object to this pool."""
+        """Add a ``PoolListener``-like object to this pool.
+
+        ``listener`` may be an object that implements some or all of
+        PoolListener, or a dictionary of callables containing implementations
+        of some or all of the named methods in PoolListener.
+
+        """
+
+        listener = as_interface(
+            listener, methods=('connect', 'checkout', 'checkin'))
 
         self.listeners.append(listener)
         if hasattr(listener, 'connect'):
@@ -278,7 +294,8 @@ def _finalize_fairy(connection, connection_record, pool, ref=None):
         return
     if connection is not None:
         try:
-            connection.rollback()
+            if pool._reset_on_return:
+                connection.rollback()
             # Immediately close detached instances
             if connection_record is None:
                 connection.close()
@@ -430,7 +447,12 @@ class _CursorFairy(object):
         try:
             self.cursor.close()
         except Exception, e:
-            self.__parent._logger.warn("Error closing cursor: " + e)
+            try:
+                ex_text = str(e)
+            except TypeError:
+                ex_text = repr(e)
+            self.__parent._logger.warn("Error closing cursor: " + ex_text)
+
             if isinstance(e, (SystemExit, KeyboardInterrupt)):
                 raise
 
@@ -655,6 +677,10 @@ class StaticPool(Pool):
     def status(self):
         return "StaticPool"
 
+    def dispose(self):
+        self._conn.close()
+        self._conn = None
+        
     def create_connection(self):
         return self._conn
 
