@@ -4,10 +4,11 @@
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
 
-import itertools, sys, warnings, sets, weakref
+import inspect, itertools, new, operator, sets, sys, warnings, weakref
 import __builtin__
+types = __import__('types')
 
-from sqlalchemy import exceptions, logging
+from sqlalchemy import exceptions
 
 try:
     import thread, threading
@@ -19,8 +20,49 @@ try:
     Set = set
     set_types = set, sets.Set
 except NameError:
-    Set = sets.Set
     set_types = sets.Set,
+    # layer some of __builtin__.set's binop behavior onto sets.Set
+    class Set(sets.Set):
+        def _binary_sanity_check(self, other):
+            pass
+
+        def issubset(self, iterable):
+            other = type(self)(iterable)
+            return sets.Set.issubset(self, other)
+        def __le__(self, other):
+            sets.Set._binary_sanity_check(self, other)
+            return sets.Set.__le__(self, other)
+        def issuperset(self, iterable):
+            other = type(self)(iterable)
+            return sets.Set.issuperset(self, other)
+        def __ge__(self, other):
+            sets.Set._binary_sanity_check(self, other)
+            return sets.Set.__ge__(self, other)
+
+        # lt and gt still require a BaseSet
+        def __lt__(self, other):
+            sets.Set._binary_sanity_check(self, other)
+            return sets.Set.__lt__(self, other)
+        def __gt__(self, other):
+            sets.Set._binary_sanity_check(self, other)
+            return sets.Set.__gt__(self, other)
+
+        def __ior__(self, other):
+            if not isinstance(other, sets.BaseSet):
+                return NotImplemented
+            return sets.Set.__ior__(self, other)
+        def __iand__(self, other):
+            if not isinstance(other, sets.BaseSet):
+                return NotImplemented
+            return sets.Set.__iand__(self, other)
+        def __ixor__(self, other):
+            if not isinstance(other, sets.BaseSet):
+                return NotImplemented
+            return sets.Set.__ixor__(self, other)
+        def __isub__(self, other):
+            if not isinstance(other, sets.BaseSet):
+                return NotImplemented
+            return sets.Set.__isub__(self, other)
 
 try:
     import cPickle as pickle
@@ -45,9 +87,8 @@ try:
 except ImportError:
     def Decimal(arg):
         if Decimal.warn:
-            warnings.warn(RuntimeWarning(
-                "True Decimal types not available on this Python, "
-                "falling back to floats."))
+            warn("True Decimal types not available on this Python, "
+                "falling back to floats.")
             Decimal.warn = False
         return float(arg)
     Decimal.warn = True
@@ -84,6 +125,61 @@ else:
                 self[key] = value = self.creator(key)
                 return value
 
+try:
+    from collections import defaultdict
+except ImportError:
+    class defaultdict(dict):
+        def __init__(self, default_factory=None, *a, **kw):
+            if (default_factory is not None and
+                not hasattr(default_factory, '__call__')):
+                raise TypeError('first argument must be callable')
+            dict.__init__(self, *a, **kw)
+            self.default_factory = default_factory
+        def __getitem__(self, key):
+            try:
+                return dict.__getitem__(self, key)
+            except KeyError:
+                return self.__missing__(key)
+        def __missing__(self, key):
+            if self.default_factory is None:
+                raise KeyError(key)
+            self[key] = value = self.default_factory()
+            return value
+        def __reduce__(self):
+            if self.default_factory is None:
+                args = tuple()
+            else:
+                args = self.default_factory,
+            return type(self), args, None, None, self.iteritems()
+        def copy(self):
+            return self.__copy__()
+        def __copy__(self):
+            return type(self)(self.default_factory, self)
+        def __deepcopy__(self, memo):
+            import copy
+            return type(self)(self.default_factory,
+                              copy.deepcopy(self.items()))
+        def __repr__(self):
+            return 'defaultdict(%s, %s)' % (self.default_factory,
+                                            dict.__repr__(self))
+
+try:
+    from collections import deque
+except ImportError:
+    class deque(list):
+        def appendleft(self, x):
+            self.insert(0, x)
+        
+        def extendleft(self, iterable):
+            self[0:0] = list(iterable)
+
+        def popleft(self):
+            return self.pop(0)
+            
+        def rotate(self, n):
+            for i in xrange(n):
+                self.appendleft(self.pop())
+                
 def to_list(x, default=None):
     if x is None:
         return default
@@ -92,6 +188,18 @@ def to_list(x, default=None):
     else:
         return x
 
+def array_as_starargs_decorator(func):
+    """Interpret a single positional array argument as
+    *args for the decorated method.
+    
+    """
+    def starargs_as_list(self, *args, **kwargs):
+        if len(args) == 1:
+            return func(self, *to_list(args[0], []), **kwargs)
+        else:
+            return func(self, *args, **kwargs)
+    return starargs_as_list
+    
 def to_set(x):
     if x is None:
         return Set()
@@ -100,6 +208,16 @@ def to_set(x):
     else:
         return x
 
+def to_ascii(x):
+    """Convert Unicode or a string with unknown encoding into ASCII."""
+
+    if isinstance(x, str):
+        return x.encode('string_escape')
+    elif isinstance(x, unicode):
+        return x.encode('unicode_escape')
+    else:
+        raise TypeError
+
 def flatten_iterator(x):
     """Given an iterator of which further sub-elements may also be
     iterators, flatten the sub-elements into a single iterator.
@@ -131,20 +249,45 @@ class ArgSingleton(type):
             return instance
 
 def get_cls_kwargs(cls):
-    """Return the full set of legal kwargs for the given `cls`."""
+    """Return the full set of inherited kwargs for the given `cls`.
+
+    Probes a class's __init__ method, collecting all named arguments.  If the
+    __init__ defines a **kwargs catch-all, then the constructor is presumed to
+    pass along unrecognized keywords to it's base classes, and the collection
+    process is repeated recursively on each of the bases.
+    """
 
-    kw = []
     for c in cls.__mro__:
-        cons = c.__init__
-        if hasattr(cons, 'func_code'):
-            for vn in cons.func_code.co_varnames:
-                if vn != 'self':
-                    kw.append(vn)
-    return kw
+        if '__init__' in c.__dict__:
+            stack = Set([c])
+            break
+    else:
+        return []
+
+    args = Set()
+    while stack:
+        class_ = stack.pop()
+        ctr = class_.__dict__.get('__init__', False)
+        if not ctr or not isinstance(ctr, types.FunctionType):
+            continue
+        names, _, has_kw, _ = inspect.getargspec(ctr)
+        args.update(names)
+        if has_kw:
+            stack.update(class_.__bases__)
+    args.discard('self')
+    return list(args)
 
 def get_func_kwargs(func):
     """Return the full set of legal kwargs for the given `func`."""
-    return [vn for vn in func.func_code.co_varnames]
+    return inspect.getargspec(func)[0]
+
+def unbound_method_to_callable(func_or_cls):
+    """Adjust the incoming callable such that a 'self' argument is not required."""
+    
+    if isinstance(func_or_cls, types.MethodType) and not func_or_cls.im_self:
+        return func_or_cls.im_func
+    else:
+        return func_or_cls
 
 # from paste.deploy.converters
 def asbool(obj):
@@ -177,11 +320,16 @@ def duck_type_collection(specimen, default=None):
     """
 
     if hasattr(specimen, '__emulates__'):
-        return specimen.__emulates__
+        # canonicalize set vs sets.Set to a standard: util.Set
+        if (specimen.__emulates__ is not None and
+            issubclass(specimen.__emulates__, set_types)):
+            return Set
+        else:
+            return specimen.__emulates__
 
     isa = isinstance(specimen, type) and issubclass or isinstance
     if isa(specimen, list): return list
-    if isa(specimen, Set): return Set
+    if isa(specimen, set_types): return Set
     if isa(specimen, dict): return dict
 
     if hasattr(specimen, 'append'):
@@ -231,7 +379,36 @@ def warn_exception(func, *args, **kwargs):
     try:
         return func(*args, **kwargs)
     except:
-        warnings.warn(RuntimeWarning("%s('%s') ignored" % sys.exc_info()[0:2]))
+        warn("%s('%s') ignored" % sys.exc_info()[0:2])
+
+def monkeypatch_proxied_specials(into_cls, from_cls, skip=None, only=None,
+                                 name='self.proxy', from_instance=None):
+    """Automates delegation of __specials__ for a proxying type."""
+
+    if only:
+        dunders = only
+    else:
+        if skip is None:
+            skip = ('__slots__', '__del__', '__getattribute__',
+                    '__metaclass__', '__getstate__', '__setstate__')
+        dunders = [m for m in dir(from_cls)
+                   if (m.startswith('__') and m.endswith('__') and
+                       not hasattr(into_cls, m) and m not in skip)]
+    for method in dunders:
+        try:
+            spec = inspect.getargspec(getattr(from_cls, method))
+            fn_args = inspect.formatargspec(spec[0])
+            d_args = inspect.formatargspec(spec[0][1:])
+        except TypeError:
+            fn_args = '(self, *args, **kw)'
+            d_args = '(*args, **kw)'
+
+        py = ("def %(method)s%(fn_args)s: "
+              "return %(name)s.%(method)s%(d_args)s" % locals())
+
+        env = from_instance is not None and {name: from_instance} or {}
+        exec py in env
+        setattr(into_cls, method, env[method])
 
 class SimpleProperty(object):
     """A *default* property accessor."""
@@ -251,6 +428,7 @@ class SimpleProperty(object):
         else:
             return getattr(obj, self.key)
 
+
 class NotImplProperty(object):
   """a property that raises ``NotImplementedError``."""
 
@@ -318,7 +496,10 @@ class OrderedProperties(object):
 
     def __contains__(self, key):
         return key in self._data
-
+    
+    def update(self, value):
+        self._data.update(value)
+        
     def get(self, key, default=None):
         if key in self:
             return self[key]
@@ -340,7 +521,8 @@ class OrderedDict(dict):
     def __init__(self, ____sequence=None, **kwargs):
         self._list = []
         if ____sequence is None:
-            self.update(**kwargs)
+            if kwargs:
+                self.update(**kwargs)
         else:
             self.update(____sequence, **kwargs)
 
@@ -348,6 +530,9 @@ class OrderedDict(dict):
         self._list = []
         dict.clear(self)
 
+    def sort(self, fn=None):
+        self._list.sort(fn)
+            
     def update(self, ____sequence=None, **kwargs):
         if ____sequence is not None:
             if hasattr(____sequence, 'keys'):
@@ -530,6 +715,14 @@ class OrderedSet(Set):
 
     __isub__ = difference_update
 
+    if hasattr(Set, '__getstate__'):
+        def __getstate__(self):
+            base = Set.__getstate__(self)
+            return base, self._list
+
+        def __setstate__(self, state):
+            Set.__setstate__(self, state[0])
+            self._list = state[1]
 
 class IdentitySet(object):
     """A set that considers only object id() for uniqueness.
@@ -538,9 +731,10 @@ class IdentitySet(object):
     two 'foo' strings in one of these sets, for example.  Use sparingly.
     """
 
+    _working_set = Set
+
     def __init__(self, iterable=None):
         self._members = _IterableUpdatableDict()
-        self._tempset = Set
         if iterable:
             for o in iterable:
                 self.add(o)
@@ -596,12 +790,12 @@ class IdentitySet(object):
         return True
 
     def __le__(self, other):
-        if not isinstance(other, set_types + (IdentitySet,)):
+        if not isinstance(other, IdentitySet):
             return NotImplemented
         return self.issubset(other)
 
     def __lt__(self, other):
-        if not isinstance(other, set_types + (IdentitySet,)):
+        if not isinstance(other, IdentitySet):
             return NotImplemented
         return len(self) < len(other) and self.issubset(other)
 
@@ -617,12 +811,12 @@ class IdentitySet(object):
         return True
 
     def __ge__(self, other):
-        if not isinstance(other, set_types + (IdentitySet,)):
+        if not isinstance(other, IdentitySet):
             return NotImplemented
         return self.issuperset(other)
 
     def __gt__(self, other):
-        if not isinstance(other, set_types + (IdentitySet,)):
+        if not isinstance(other, IdentitySet):
             return NotImplemented
         return len(self) > len(other) and self.issuperset(other)
 
@@ -630,20 +824,19 @@ class IdentitySet(object):
         result = type(self)()
         # testlib.pragma exempt:__hash__
         result._members.update(
-            self._tempset(self._members.iteritems()).union(_iter_id(iterable)))
+            self._working_set(self._members.iteritems()).union(_iter_id(iterable)))
         return result
 
     def __or__(self, other):
-        if not isinstance(other, set_types + (IdentitySet,)):
+        if not isinstance(other, IdentitySet):
             return NotImplemented
         return self.union(other)
-    __ror__ = __or__
 
     def update(self, iterable):
         self._members = self.union(iterable)._members
 
     def __ior__(self, other):
-        if not isinstance(other, set_types + (IdentitySet,)):
+        if not isinstance(other, IdentitySet):
             return NotImplemented
         self.update(other)
         return self
@@ -652,20 +845,19 @@ class IdentitySet(object):
         result = type(self)()
         # testlib.pragma exempt:__hash__
         result._members.update(
-            self._tempset(self._members.iteritems()).difference(_iter_id(iterable)))
+            self._working_set(self._members.iteritems()).difference(_iter_id(iterable)))
         return result
 
     def __sub__(self, other):
-        if not isinstance(other, set_types + (IdentitySet,)):
+        if not isinstance(other, IdentitySet):
             return NotImplemented
         return self.difference(other)
-    __rsub__ = __sub__
 
     def difference_update(self, iterable):
         self._members = self.difference(iterable)._members
 
     def __isub__(self, other):
-        if not isinstance(other, set_types + (IdentitySet,)):
+        if not isinstance(other, IdentitySet):
             return NotImplemented
         self.difference_update(other)
         return self
@@ -674,20 +866,19 @@ class IdentitySet(object):
         result = type(self)()
         # testlib.pragma exempt:__hash__
         result._members.update(
-            self._tempset(self._members.iteritems()).intersection(_iter_id(iterable)))
+            self._working_set(self._members.iteritems()).intersection(_iter_id(iterable)))
         return result
 
     def __and__(self, other):
-        if not isinstance(other, set_types + (IdentitySet,)):
+        if not isinstance(other, IdentitySet):
             return NotImplemented
         return self.intersection(other)
-    __rand__ = __and__
 
     def intersection_update(self, iterable):
         self._members = self.intersection(iterable)._members
 
     def __iand__(self, other):
-        if not isinstance(other, set_types + (IdentitySet,)):
+        if not isinstance(other, IdentitySet):
             return NotImplemented
         self.intersection_update(other)
         return self
@@ -696,20 +887,19 @@ class IdentitySet(object):
         result = type(self)()
         # testlib.pragma exempt:__hash__
         result._members.update(
-            self._tempset(self._members.iteritems()).symmetric_difference(_iter_id(iterable)))
+            self._working_set(self._members.iteritems()).symmetric_difference(_iter_id(iterable)))
         return result
 
     def __xor__(self, other):
-        if not isinstance(other, set_types + (IdentitySet,)):
+        if not isinstance(other, IdentitySet):
             return NotImplemented
         return self.symmetric_difference(other)
-    __rxor__ = __xor__
 
     def symmetric_difference_update(self, iterable):
         self._members = self.symmetric_difference(iterable)._members
 
     def __ixor__(self, other):
-        if not isinstance(other, set_types + (IdentitySet,)):
+        if not isinstance(other, IdentitySet):
             return NotImplemented
         self.symmetric_difference(other)
         return self
@@ -751,15 +941,20 @@ def _iter_id(iterable):
 
 
 class OrderedIdentitySet(IdentitySet):
+    class _working_set(OrderedSet):
+        # a testing pragma: exempt the OIDS working set from the test suite's
+        # "never call the user's __hash__" assertions.  this is a big hammer,
+        # but it's safe here: IDS operates on (id, instance) tuples in the
+        # working set.
+        __sa_hash_exempt__ = True
+
     def __init__(self, iterable=None):
         IdentitySet.__init__(self)
         self._members = OrderedDict()
-        self._tempset = OrderedSet
         if iterable:
             for o in iterable:
                 self.add(o)
 
-
 class UniqueAppender(object):
     """Only adds items to a collection once.
 
@@ -829,20 +1024,263 @@ class ScopedRegistry(object):
     def _get_key(self):
         return self.scopefunc()
 
+class _symbol(object):
+    def __init__(self, name):
+        """Construct a new named symbol."""
+        assert isinstance(name, str)
+        self.name = name
+    def __reduce__(self):
+        return symbol, (self.name,)
+    def __repr__(self):
+        return "<symbol '%s>" % self.name
+_symbol.__name__ = 'symbol'
 
-def warn_deprecated(msg):
-    warnings.warn(logging.SADeprecationWarning(msg), stacklevel=3)
+class symbol(object):
+    """A constant symbol.
 
-def deprecated(func, add_deprecation_to_docstring=True):
-    def func_with_warning(*args, **kwargs):
-        warnings.warn(logging.SADeprecationWarning("Call to deprecated function %s" % func.__name__),
-                      stacklevel=2)
-        return func(*args, **kwargs)
-    func_with_warning.__doc__ = (add_deprecation_to_docstring and 'Deprecated.\n' or '') + str(func.__doc__)
-    func_with_warning.__dict__.update(func.__dict__)
+    >>> symbol('foo') is symbol('foo')
+    True
+    >>> symbol('foo')
+    <symbol 'foo>
+
+    A slight refinement of the MAGICCOOKIE=object() pattern.  The primary
+    advantage of symbol() is its repr().  They are also singletons.
+
+    Repeated calls of symbol('name') will all return the same instance.
+
+    """
+    symbols = {}
+    _lock = threading.Lock()
+
+    def __new__(cls, name):
+        cls._lock.acquire()
+        try:
+            sym = cls.symbols.get(name)
+            if sym is None:
+                cls.symbols[name] = sym = _symbol(name)
+            return sym
+        finally:
+            symbol._lock.release()
+
+
+def as_interface(obj, cls=None, methods=None, required=None):
+    """Ensure basic interface compliance for an instance or dict of callables.
+
+    Checks that ``obj`` implements public methods of ``cls`` or has members
+    listed in ``methods``.  If ``required`` is not supplied, implementing at
+    least one interface method is sufficient.  Methods present on ``obj`` that
+    are not in the interface are ignored.
+
+    If ``obj`` is a dict and ``dict`` does not meet the interface
+    requirements, the keys of the dictionary are inspected. Keys present in
+    ``obj`` that are not in the interface will raise TypeErrors.
+
+    Raises TypeError if ``obj`` does not meet the interface criteria.
+
+    In all passing cases, an object with callable members is returned.  In the
+    simple case, ``obj`` is returned as-is; if dict processing kicks in then
+    an anonymous class is returned.
+
+    obj
+      A type, instance, or dictionary of callables.
+    cls
+      Optional, a type.  All public methods of cls are considered the
+      interface.  An ``obj`` instance of cls will always pass, ignoring
+      ``required``..
+    methods
+      Optional, a sequence of method names to consider as the interface.
+    required
+      Optional, a sequence of mandatory implementations. If omitted, an
+      ``obj`` that provides at least one interface method is considered
+      sufficient.  As a convenience, required may be a type, in which case
+      all public methods of the type are required.
+
+    """
+    if not cls and not methods:
+        raise TypeError('a class or collection of method names are required')
+
+    if isinstance(cls, type) and isinstance(obj, cls):
+        return obj
+
+    interface = Set(methods or [m for m in dir(cls) if not m.startswith('_')])
+    implemented = Set(dir(obj))
+
+    complies = operator.ge
+    if isinstance(required, type):
+        required = interface
+    elif not required:
+        required = Set()
+        complies = operator.gt
+    else:
+        required = Set(required)
+
+    if complies(implemented.intersection(interface), required):
+        return obj
+
+    # No dict duck typing here.
+    if not type(obj) is dict:
+        qualifier = complies is operator.gt and 'any of' or 'all of'
+        raise TypeError("%r does not implement %s: %s" % (
+            obj, qualifier, ', '.join(interface)))
+
+    class AnonymousInterface(object):
+        """A callable-holding shell."""
+
+    if cls:
+        AnonymousInterface.__name__ = 'Anonymous' + cls.__name__
+    found = Set()
+
+    for method, impl in dictlike_iteritems(obj):
+        if method not in interface:
+            raise TypeError("%r: unknown in this interface" % method)
+        if not callable(impl):
+            raise TypeError("%r=%r is not callable" % (method, impl))
+        setattr(AnonymousInterface, method, staticmethod(impl))
+        found.add(method)
+
+    if complies(found, required):
+        return AnonymousInterface
+
+    raise TypeError("dictionary does not contain required keys %s" %
+                    ', '.join(required - found))
+
+def function_named(fn, name):
+    """Return a function with a given __name__.
+
+    Will assign to __name__ and return the original function if possible on
+    the Python implementation, otherwise a new function will be constructed.
+
+    """
     try:
-        func_with_warning.__name__ = func.__name__
+        fn.__name__ = name
     except TypeError:
+        fn = new.function(fn.func_code, fn.func_globals, name,
+                          fn.func_defaults, fn.func_closure)
+    return fn
+
+
+_creation_order = 1 
+def set_creation_order(instance): 
+    """assign a '_creation_order' sequence to the given instance. 
+ 
+    This allows multiple instances to be sorted in order of 
+    creation (typically within a single thread; the counter is 
+    not particularly threadsafe). 
+
+    """ 
+    global _creation_order 
+    instance._creation_order = _creation_order 
+    _creation_order +=1
+
+def conditional_cache_decorator(func):
+    """apply conditional caching to the return value of a function."""
+
+    return cache_decorator(func, conditional=True)
+
+def cache_decorator(func, conditional=False):
+    """apply caching to the return value of a function."""
+
+    name = '_cached_' + func.__name__
+    
+    def do_with_cache(self, *args, **kwargs):
+        if conditional:
+            cache = kwargs.pop('cache', False)
+            if not cache:
+                return func(self, *args, **kwargs)
+        try:
+            return getattr(self, name)
+        except AttributeError:
+            value = func(self, *args, **kwargs)
+            setattr(self, name, value)
+            return value
+    return do_with_cache
+    
+def reset_cached(instance, name):
+    try:
+        delattr(instance, '_cached_' + name)
+    except AttributeError:
         pass
 
-    return func_with_warning
+def warn(msg):
+    if isinstance(msg, basestring):
+        warnings.warn(msg, exceptions.SAWarning, stacklevel=3)
+    else:
+        warnings.warn(msg, stacklevel=3)
+
+def warn_deprecated(msg):
+    warnings.warn(msg, exceptions.SADeprecationWarning, stacklevel=3)
+
+def deprecated(message=None, add_deprecation_to_docstring=True):
+    """Decorates a function and issues a deprecation warning on use.
+
+    message
+      If provided, issue message in the warning.  A sensible default
+      is used if not provided.
+
+    add_deprecation_to_docstring
+      Default True.  If False, the wrapped function's __doc__ is left
+      as-is.  If True, the 'message' is prepended to the docs if
+      provided, or sensible default if message is omitted.
+    """
+
+    if add_deprecation_to_docstring:
+        header = message is not None and message or 'Deprecated.'
+    else:
+        header = None
+
+    if message is None:
+        message = "Call to deprecated function %(func)s"
+
+    def decorate(fn):
+        return _decorate_with_warning(
+            fn, exceptions.SADeprecationWarning,
+            message % dict(func=fn.__name__), header)
+    return decorate
+
+def pending_deprecation(version, message=None,
+                        add_deprecation_to_docstring=True):
+    """Decorates a function and issues a pending deprecation warning on use.
+
+    version
+      An approximate future version at which point the pending deprecation
+      will become deprecated.  Not used in messaging.
+
+    message
+      If provided, issue message in the warning.  A sensible default
+      is used if not provided.
+
+    add_deprecation_to_docstring
+      Default True.  If False, the wrapped function's __doc__ is left
+      as-is.  If True, the 'message' is prepended to the docs if
+      provided, or sensible default if message is omitted.
+    """
+
+    if add_deprecation_to_docstring:
+        header = message is not None and message or 'Deprecated.'
+    else:
+        header = None
+
+    if message is None:
+        message = "Call to deprecated function %(func)s"
+
+    def decorate(fn):
+        return _decorate_with_warning(
+            fn, exceptions.SAPendingDeprecationWarning,
+            message % dict(func=fn.__name__), header)
+    return decorate
+
+def _decorate_with_warning(func, wtype, message, docstring_header=None):
+    """Wrap a function with a warnings.warn and augmented docstring."""
+
+    def func_with_warning(*args, **kwargs):
+        warnings.warn(wtype(message), stacklevel=2)
+        return func(*args, **kwargs)
+
+    doc = func.__doc__ is not None and func.__doc__ or ''
+    if docstring_header is not None:
+        doc = '\n'.join((docstring_header.rstrip(), doc))
+
+    func_with_warning.__doc__ = doc
+    func_with_warning.__dict__.update(func.__dict__)
+
+    return function_named(func_with_warning, func.__name__)
