@@ -1,10 +1,12 @@
-import testbase
+import testenv; testenv.configure_for_tests()
 import unittest
 
 def suite():
     modules_to_test = (
         # core utilities
         'base.dependency',
+        'base.utils',
+        'base.except',
         )
     alltests = unittest.TestSuite()
     for name in modules_to_test:
@@ -16,4 +18,4 @@ def suite():
 
 
 if __name__ == '__main__':
-    testbase.main(suite())
+    testenv.main(suite())
