@@ -1,9 +1,10 @@
-import testbase
+import testenv; testenv.configure_for_tests()
 from sqlalchemy import *
 from sqlalchemy.databases import sybase
 from testlib import *
 
-class BasicTest(AssertMixin):
+
+class BasicTest(TestBase, AssertsExecutionResults):
     # A simple import of the database/ module should work on all systems.
     def test_import(self):
         # we got this far, right?
@@ -11,4 +12,4 @@ class BasicTest(AssertMixin):
 
 
 if __name__ == "__main__":
-    testbase.main()
+    testenv.main()
