@@ -1,4 +1,4 @@
-import testbase
+import testenv; testenv.configure_for_tests()
 import threading, thread, time, gc
 import sqlalchemy.pool as pool
 import sqlalchemy.interfaces as interfaces
@@ -33,7 +33,7 @@ class MockCursor(object):
         pass
 mock_dbapi = MockDBAPI()
 
-class PoolTest(PersistTest):
+class PoolTest(TestBase):
 
     def setUp(self):
         pool.clear_managers()
@@ -131,11 +131,13 @@ class PoolTest(PersistTest):
             assert int(time.time() - now) == 2
 
     def test_timeout_race(self):
-        # test a race condition where the initial connecting threads all race to queue.Empty, then block on the mutex.
-        # each thread consumes a connection as they go in.  when the limit is reached, the remaining threads
-        # go in, and get TimeoutError; even though they never got to wait for the timeout on queue.get().
-        # the fix involves checking the timeout again within the mutex, and if so, unlocking and throwing them back to the start
-        # of do_get()
+        # test a race condition where the initial connecting threads all race
+        # to queue.Empty, then block on the mutex.  each thread consumes a
+        # connection as they go in.  when the limit is reached, the remaining
+        # threads go in, and get TimeoutError; even though they never got to
+        # wait for the timeout on queue.get().  the fix involves checking the
+        # timeout again within the mutex, and if so, unlocking and throwing
+        # them back to the start of do_get()
         p = pool.QueuePool(creator = lambda: mock_dbapi.connect('foo.db', delay=.05), pool_size = 2, max_overflow = 1, use_threadlocal = False, timeout=3)
         timeouts = []
         def checkout():
@@ -451,12 +453,12 @@ class PoolTest(PersistTest):
             def checkout(self, con, record, proxy, num):
                 pass
         class ListenCheckIn(InstrumentingListener):
-            def checkin(self, con, proxy, record):
+            def checkin(self, con, record):
                 pass
 
         def _pool(**kw):
-            return pool.QueuePool(creator=lambda: dbapi.connect('foo.db'), use_threadlocal=False, **kw)
-            #, pool_size=1, max_overflow=0, **kw)
+            return pool.QueuePool(creator=lambda: dbapi.connect('foo.db'),
+                                  use_threadlocal=False, **kw)
 
         def assert_listeners(p, total, conn, cout, cin):
             for instance in (p, p.recreate()):
@@ -549,9 +551,68 @@ class PoolTest(PersistTest):
         del c
         snoop.assert_total(2, 2, 1)
 
+    def test_listeners_callables(self):
+        dbapi = MockDBAPI()
+
+        counts = [0, 0, 0]
+        def connect(dbapi_con, con_record):
+            counts[0] += 1
+        def checkout(dbapi_con, con_record, con_proxy):
+            counts[1] += 1
+        def checkin(dbapi_con, con_record):
+            counts[2] += 1
+
+        i_all = dict(connect=connect, checkout=checkout, checkin=checkin)
+        i_connect = dict(connect=connect)
+        i_checkout = dict(checkout=checkout)
+        i_checkin = dict(checkin=checkin)
+
+        def _pool(**kw):
+            return pool.QueuePool(creator=lambda: dbapi.connect('foo.db'),
+                                  use_threadlocal=False, **kw)
+
+        def assert_listeners(p, total, conn, cout, cin):
+            for instance in (p, p.recreate()):
+                self.assert_(len(instance.listeners) == total)
+                self.assert_(len(instance._on_connect) == conn)
+                self.assert_(len(instance._on_checkout) == cout)
+                self.assert_(len(instance._on_checkin) == cin)
+
+        p = _pool()
+        assert_listeners(p, 0, 0, 0, 0)
+
+        p.add_listener(i_all)
+        assert_listeners(p, 1, 1, 1, 1)
+
+        p.add_listener(i_connect)
+        assert_listeners(p, 2, 2, 1, 1)
+
+        p.add_listener(i_checkout)
+        assert_listeners(p, 3, 2, 2, 1)
+
+        p.add_listener(i_checkin)
+        assert_listeners(p, 4, 2, 2, 2)
+        del p
+
+        p = _pool(listeners=[i_all])
+        assert_listeners(p, 1, 1, 1, 1)
+
+        c = p.connect()
+        assert counts == [1, 1, 0]
+        c.close()
+        assert counts == [1, 1, 1]
+
+        c = p.connect()
+        assert counts == [1, 2, 1]
+        p.add_listener(i_checkin)
+        c.close()
+        assert counts == [1, 2, 3]
+
+
+
     def tearDown(self):
        pool.clear_managers()
 
 
 if __name__ == "__main__":
-    testbase.main()
+    testenv.main()
