@@ -1,13 +1,17 @@
-import testbase
-import unittest, doctest
+import testenv; testenv.configure_for_tests()
+import doctest, sys, unittest
 
 def suite():
     unittest_modules = ['ext.activemapper',
-                        'ext.selectresults',
                         'ext.assignmapper',
+                        'ext.declarative',
                         'ext.orderinglist',
                         'ext.associationproxy']
-    doctest_modules = ['sqlalchemy.ext.sqlsoup']
+
+    if sys.version_info >= (2, 4):
+        doctest_modules = ['sqlalchemy.ext.sqlsoup']
+    else:
+        doctest_modules = []
 
     alltests = unittest.TestSuite()
     for name in unittest_modules:
@@ -21,4 +25,4 @@ def suite():
 
 
 if __name__ == '__main__':
-    testbase.main(suite())
+    testenv.main(suite())
