@@ -1,5 +1,5 @@
-import testbase
-
+import testenv; testenv.configure_for_tests()
+import gc
 from sqlalchemy import *
 from sqlalchemy.orm import *
 from sqlalchemy.orm.collections import collection
@@ -33,11 +33,11 @@ class ObjectCollection(object):
     def __iter__(self):
         return iter(self.values)
 
-class _CollectionOperations(PersistTest):
+class _CollectionOperations(TestBase):
     def setUp(self):
         collection_class = self.collection_class
 
-        metadata = MetaData(testbase.db)
+        metadata = MetaData(testing.db)
 
         parents_table = Table('Parent', metadata,
                               Column('id', Integer, primary_key=True),
@@ -199,13 +199,18 @@ class _CollectionOperations(PersistTest):
         self.assert_(p1.children == after)
         self.assert_([c.name for c in p1._children] == after)
 
+        p1.children += ['c']
+        after = ['a', 'b', 'c']
+        self.assert_(p1.children == after)
+        self.assert_([c.name for c in p1._children] == after)
+
         p1.children *= 1
-        after = ['a', 'b']
+        after = ['a', 'b', 'c']
         self.assert_(p1.children == after)
         self.assert_([c.name for c in p1._children] == after)
 
         p1.children *= 2
-        after = ['a', 'b', 'a', 'b']
+        after = ['a', 'b', 'c', 'a', 'b', 'c']
         self.assert_(p1.children == after)
         self.assert_([c.name for c in p1._children] == after)
 
@@ -219,8 +224,8 @@ class _CollectionOperations(PersistTest):
         self.assert_((p1.children * 0) == [])
         self.assert_((0 * p1.children) == [])
 
-        self.assert_((p1.children + ['a']) == ['a', 'a'])
-        self.assert_((['a'] + p1.children) == ['a', 'a'])
+        self.assert_((p1.children + ['b']) == ['a', 'b'])
+        self.assert_((['b'] + p1.children) == ['b', 'a'])
 
         try:
             p1.children + 123
@@ -313,7 +318,7 @@ class CustomDictTest(DictTest):
         self.assert_(len(p1._children) == 3)
         self.assert_(len(p1.children) == 3)
 
-        p1.children['e'] = 'changed-in-place' 
+        p1.children['e'] = 'changed-in-place'
         self.assert_(p1.children['e'] == 'changed-in-place')
         inplace_id = p1._children['e'].id
         p1 = self.roundtrip(p1)
@@ -529,6 +534,7 @@ class SetTest(_CollectionOperations):
                 for other in (set(['a','b','c']), set(['a','b','c','d']),
                               set(['a']), set(['a','b']),
                               set(['c','d']), set(['e', 'f', 'g']),
+                              frozenset(['e', 'f', 'g']),
                               set()):
                     p = Parent('p')
                     p.children = base[:]
@@ -586,9 +592,9 @@ class CustomObjectTest(_CollectionOperations):
         except TypeError:
             pass
 
-class ScalarTest(PersistTest):
+class ScalarTest(TestBase):
     def test_scalar_proxy(self):
-        metadata = MetaData(testbase.db)
+        metadata = MetaData(testing.db)
 
         parents_table = Table('Parent', metadata,
                               Column('id', Integer, primary_key=True),
@@ -703,9 +709,9 @@ class ScalarTest(PersistTest):
         p2.bar = 'quux'
 
 
-class LazyLoadTest(PersistTest):
+class LazyLoadTest(TestBase):
     def setUp(self):
-        metadata = MetaData(testbase.db)
+        metadata = MetaData(testing.db)
 
         parents_table = Table('Parent', metadata,
                               Column('id', Integer, primary_key=True),
@@ -810,5 +816,64 @@ class LazyLoadTest(PersistTest):
         self.assert_(p._children is not None)
 
 
+class ReconstitutionTest(TestBase):
+    def setUp(self):
+        metadata = MetaData(testing.db)
+        parents = Table('parents', metadata,
+                        Column('id', Integer, primary_key=True),
+                        Column('name', String(30)))
+        children = Table('children', metadata,
+                         Column('id', Integer, primary_key=True),
+                         Column('parent_id', Integer, ForeignKey('parents.id')),
+                         Column('name', String(30)))
+        metadata.create_all()
+        parents.insert().execute(name='p1')
+
+        class Parent(object):
+            kids = association_proxy('children', 'name')
+            def __init__(self, name):
+                self.name = name
+
+        class Child(object):
+            def __init__(self, name):
+                self.name = name
+
+        mapper(Parent, parents, properties=dict(children=relation(Child)))
+        mapper(Child, children)
+
+        self.metadata = metadata
+        self.Parent = Parent
+
+    def tearDown(self):
+        self.metadata.drop_all()
+
+    def test_weak_identity_map(self):
+        session = create_session(weak_identity_map=True)
+
+        def add_child(parent_name, child_name):
+            parent = (session.query(self.Parent).
+                      filter_by(name=parent_name)).one()
+            parent.kids.append(child_name)
+
+
+        add_child('p1', 'c1')
+        gc.collect()
+        add_child('p1', 'c2')
+
+        session.flush()
+        p = session.query(self.Parent).filter_by(name='p1').one()
+        assert set(p.kids) == set(['c1', 'c2']), p.kids
+
+    def test_copy(self):
+        import copy
+        p = self.Parent('p1')
+        p.kids.extend(['c1', 'c2'])
+        p_copy = copy.copy(p)
+        del p
+        gc.collect()
+
+        assert set(p_copy.kids) == set(['c1', 'c2']), p.kids
+
+
 if __name__ == "__main__":
-    testbase.main()
+    testenv.main()
