@@ -1,16 +1,20 @@
-import testbase
+import testenv; testenv.configure_for_tests()
 from sqlalchemy import *
+from sqlalchemy import exceptions
+from sqlalchemy.orm import *
+from testlib import *
 
-class CompileTest(testbase.AssertMixin):
+
+class CompileTest(TestBase, AssertsExecutionResults):
     """test various mapper compilation scenarios"""
-    def tearDownAll(self):
+    def tearDown(self):
         clear_mappers()
-        
+
     def testone(self):
         global metadata, order, employee, product, tax, orderproduct
-        metadata = MetaData(testbase.db)
+        metadata = MetaData(testing.db)
 
-        order = Table('orders', metadata, 
+        order = Table('orders', metadata,
             Column('id', Integer, primary_key=True),
             Column('employee_id', Integer, ForeignKey('employees.id'), nullable=False),
             Column('type', Unicode(16)))
@@ -43,9 +47,9 @@ class CompileTest(testbase.AssertMixin):
 
         order_join = order.select().alias('pjoin')
 
-        order_mapper = mapper(Order, order, 
-            select_table=order_join, 
-            polymorphic_on=order_join.c.type, 
+        order_mapper = mapper(Order, order,
+            select_table=order_join,
+            polymorphic_on=order_join.c.type,
             polymorphic_identity='order',
             properties={
                 'orderproducts': relation(OrderProduct, lazy=True, backref='order')}
@@ -61,7 +65,7 @@ class CompileTest(testbase.AssertMixin):
                 'orders': relation(Order, lazy=True, backref='employee')})
 
         mapper(OrderProduct, orderproduct)
-        
+
         # this requires that the compilation of order_mapper's "surrogate mapper" occur after
         # the initial setup of MapperProperty objects on the mapper.
         class_mapper(Product).compile()
@@ -69,9 +73,9 @@ class CompileTest(testbase.AssertMixin):
     def testtwo(self):
         """test that conflicting backrefs raises an exception"""
         global metadata, order, employee, product, tax, orderproduct
-        metadata = MetaData(testbase.db)
+        metadata = MetaData(testing.db)
 
-        order = Table('orders', metadata, 
+        order = Table('orders', metadata,
             Column('id', Integer, primary_key=True),
             Column('type', Unicode(16)))
 
@@ -96,9 +100,9 @@ class CompileTest(testbase.AssertMixin):
 
         order_join = order.select().alias('pjoin')
 
-        order_mapper = mapper(Order, order, 
-            select_table=order_join, 
-            polymorphic_on=order_join.c.type, 
+        order_mapper = mapper(Order, order,
+            select_table=order_join,
+            polymorphic_on=order_join.c.type,
             polymorphic_identity='order',
             properties={
                 'orderproducts': relation(OrderProduct, lazy=True, backref='product')}
@@ -115,15 +119,15 @@ class CompileTest(testbase.AssertMixin):
             class_mapper(Product).compile()
             assert False
         except exceptions.ArgumentError, e:
-            assert str(e).index("Backrefs do not match") > -1
+            assert str(e).index("Error creating backref ") > -1
 
     def testthree(self):
-        metadata = MetaData(testbase.db)
-        node_table = Table("node", metadata, 
+        metadata = MetaData(testing.db)
+        node_table = Table("node", metadata,
             Column('node_id', Integer, primary_key=True),
             Column('name_index', Integer, nullable=True),
             )
-        node_name_table = Table("node_name", metadata, 
+        node_name_table = Table("node_name", metadata,
             Column('node_name_id', Integer, primary_key=True),
             Column('node_id', Integer, ForeignKey('node.node_id')),
             Column('host_id', Integer, ForeignKey('host.host_id')),
@@ -140,7 +144,7 @@ class CompileTest(testbase.AssertMixin):
             class Node(object):pass
             class NodeName(object):pass
             class Host(object):pass
-                
+
             node_mapper = mapper(Node, node_table)
             host_mapper = mapper(Host, host_table)
             node_name_mapper = mapper(NodeName, node_name_table,
@@ -154,5 +158,27 @@ class CompileTest(testbase.AssertMixin):
         finally:
             metadata.drop_all()
 
+    def testfour(self):
+        meta = MetaData()
+
+        a = Table('a', meta, Column('id', Integer, primary_key=True))
+        b = Table('b', meta, Column('id', Integer, primary_key=True), Column('a_id', Integer, ForeignKey('a.id')))
+
+        class A(object):pass
+        class B(object):pass
+
+        mapper(A, a, properties={
+            'b':relation(B, backref='a')
+        })
+        mapper(B, b, properties={
+            'a':relation(A, backref='b')
+        })
+
+        try:
+            compile_mappers()
+            assert False
+        except exceptions.ArgumentError, e:
+            assert str(e).index("Error creating backref") > -1
+
 if __name__ == '__main__':
-    testbase.main()
+    testenv.main()
