@@ -1,4 +1,4 @@
-import testbase
+import testenv; testenv.configure_for_tests()
 from sqlalchemy import *
 from sqlalchemy.orm import *
 from sqlalchemy import exceptions
@@ -12,10 +12,10 @@ class Foo(object):
         for k in kwargs:
             setattr(self, k, kwargs[k])
 
-class GenerativeQueryTest(PersistTest):
+class GenerativeQueryTest(TestBase):
     def setUpAll(self):
         global foo, metadata
-        metadata = MetaData(testbase.db)
+        metadata = MetaData(testing.db)
         foo = Table('foo', metadata,
                     Column('id', Integer, Sequence('foo_id_seq'), primary_key=True),
                     Column('bar', Integer),
@@ -24,7 +24,7 @@ class GenerativeQueryTest(PersistTest):
         mapper(Foo, foo)
         metadata.create_all()
 
-        sess = create_session(bind=testbase.db)
+        sess = create_session(bind=testing.db)
         for i in range(100):
             sess.save(Foo(bar=i, range=i%10))
         sess.flush()
@@ -34,14 +34,14 @@ class GenerativeQueryTest(PersistTest):
         clear_mappers()
 
     def test_selectby(self):
-        res = create_session(bind=testbase.db).query(Foo).filter_by(range=5)
+        res = create_session(bind=testing.db).query(Foo).filter_by(range=5)
         assert res.order_by([Foo.c.bar])[0].bar == 5
         assert res.order_by([desc(Foo.c.bar)])[0].bar == 95
 
     @testing.unsupported('mssql')
     @testing.fails_on('maxdb')
     def test_slice(self):
-        sess = create_session(bind=testbase.db)
+        sess = create_session(bind=testing.db)
         query = sess.query(Foo)
         orig = query.all()
         assert query[1] == orig[1]
@@ -53,8 +53,9 @@ class GenerativeQueryTest(PersistTest):
         assert list(query[-5:]) == orig[-5:]
         assert query[10:20][5] == orig[10:20][5]
 
+    @testing.uses_deprecated('Call to deprecated function apply_max')
     def test_aggregate(self):
-        sess = create_session(bind=testbase.db)
+        sess = create_session(bind=testing.db)
         query = sess.query(Foo)
         assert query.count() == 100
         assert query.filter(foo.c.bar<30).min(foo.c.bar) == 0
@@ -64,38 +65,38 @@ class GenerativeQueryTest(PersistTest):
 
     def test_aggregate_1(self):
         if (testing.against('mysql') and
-            testbase.db.dialect.dbapi.version_info[:4] == (1, 2, 1, 'gamma')):
+            testing.db.dialect.dbapi.version_info[:4] == (1, 2, 1, 'gamma')):
             return
 
-        query = create_session(bind=testbase.db).query(Foo)
+        query = create_session(bind=testing.db).query(Foo)
         assert query.filter(foo.c.bar<30).sum(foo.c.bar) == 435
 
-    @testing.fails_on('postgres', 'mysql', 'firebird', 'mssql')
+    @testing.fails_on('firebird', 'mssql')
     def test_aggregate_2(self):
-        query = create_session(bind=testbase.db).query(Foo)
-        assert query.filter(foo.c.bar<30).avg(foo.c.bar) == 14.5
+        query = create_session(bind=testing.db).query(Foo)
+        avg = query.filter(foo.c.bar < 30).avg(foo.c.bar)
+        assert round(avg, 1) == 14.5
 
-    @testing.fails_on_everything_except('sqlite', 'postgres', 'mysql',
-                                        'firebird', 'mssql')
-    def test_aggregate_2_int(self):
-        query = create_session(bind=testbase.db).query(Foo)
-        assert int(query.filter(foo.c.bar<30).avg(foo.c.bar)) == 14
-
-    @testing.fails_on('postgres', 'mysql', 'firebird', 'mssql')
+    @testing.fails_on('firebird', 'mssql')
+    @testing.uses_deprecated('Call to deprecated function apply_avg')
     def test_aggregate_3(self):
-        query = create_session(bind=testbase.db).query(Foo)
-        assert query.filter(foo.c.bar<30).apply_avg(foo.c.bar).first() == 14.5
-        assert query.filter(foo.c.bar<30).apply_avg(foo.c.bar).one() == 14.5
+        query = create_session(bind=testing.db).query(Foo)
+
+        avg_f = query.filter(foo.c.bar<30).apply_avg(foo.c.bar).first()
+        assert round(avg_f, 1) == 14.5
+
+        avg_o = query.filter(foo.c.bar<30).apply_avg(foo.c.bar).one()
+        assert round(avg_o, 1) == 14.5
 
     def test_filter(self):
-        query = create_session(bind=testbase.db).query(Foo)
+        query = create_session(bind=testing.db).query(Foo)
         assert query.count() == 100
         assert query.filter(Foo.c.bar < 30).count() == 30
         res2 = query.filter(Foo.c.bar < 30).filter(Foo.c.bar > 10)
         assert res2.count() == 19
 
     def test_options(self):
-        query = create_session(bind=testbase.db).query(Foo)
+        query = create_session(bind=testing.db).query(Foo)
         class ext1(MapperExtension):
             def populate_instance(self, mapper, selectcontext, row, instance, **flags):
                 instance.TEST = "hello world"
@@ -103,16 +104,16 @@ class GenerativeQueryTest(PersistTest):
         assert query.options(extension(ext1()))[0].TEST == "hello world"
 
     def test_order_by(self):
-        query = create_session(bind=testbase.db).query(Foo)
+        query = create_session(bind=testing.db).query(Foo)
         assert query.order_by([Foo.c.bar])[0].bar == 0
         assert query.order_by([desc(Foo.c.bar)])[0].bar == 99
 
     def test_offset(self):
-        query = create_session(bind=testbase.db).query(Foo)
+        query = create_session(bind=testing.db).query(Foo)
         assert list(query.order_by([Foo.c.bar]).offset(10))[0].bar == 10
 
     def test_offset(self):
-        query = create_session(bind=testbase.db).query(Foo)
+        query = create_session(bind=testing.db).query(Foo)
         assert len(list(query.limit(10))) == 10
 
 class Obj1(object):
@@ -120,7 +121,7 @@ class Obj1(object):
 class Obj2(object):
     pass
 
-class GenerativeTest2(PersistTest):
+class GenerativeTest2(TestBase):
     def setUpAll(self):
         global metadata, table1, table2
         metadata = MetaData()
@@ -133,24 +134,24 @@ class GenerativeTest2(PersistTest):
             )
         mapper(Obj1, table1)
         mapper(Obj2, table2)
-        metadata.create_all(bind=testbase.db)
-        testbase.db.execute(table1.insert(), {'id':1},{'id':2},{'id':3},{'id':4})
-        testbase.db.execute(table2.insert(), {'num':1,'t1id':1},{'num':2,'t1id':1},{'num':3,'t1id':1},\
+        metadata.create_all(bind=testing.db)
+        testing.db.execute(table1.insert(), {'id':1},{'id':2},{'id':3},{'id':4})
+        testing.db.execute(table2.insert(), {'num':1,'t1id':1},{'num':2,'t1id':1},{'num':3,'t1id':1},\
 {'num':4,'t1id':2},{'num':5,'t1id':2},{'num':6,'t1id':3})
 
     def tearDownAll(self):
-        metadata.drop_all(bind=testbase.db)
+        metadata.drop_all(bind=testing.db)
         clear_mappers()
 
     def test_distinctcount(self):
-        query = create_session(bind=testbase.db).query(Obj1)
+        query = create_session(bind=testing.db).query(Obj1)
         assert query.count() == 4
         res = query.filter(and_(table1.c.id==table2.c.t1id,table2.c.t1id==1))
         assert res.count() == 3
         res = query.filter(and_(table1.c.id==table2.c.t1id,table2.c.t1id==1)).distinct()
         self.assertEqual(res.count(), 1)
 
-class RelationsTest(AssertMixin):
+class RelationsTest(TestBase, AssertsExecutionResults):
     def setUpAll(self):
         tables.create()
         tables.data()
@@ -165,7 +166,7 @@ class RelationsTest(AssertMixin):
                 'items':relation(mapper(tables.Item, tables.orderitems))
             }))
         })
-        session = create_session(bind=testbase.db)
+        session = create_session(bind=testing.db)
         query = session.query(tables.User)
         x = query.join(['orders', 'items']).filter(tables.Item.c.item_id==2)
         print x.compile()
@@ -177,7 +178,7 @@ class RelationsTest(AssertMixin):
                 'items':relation(mapper(tables.Item, tables.orderitems))
             }))
         })
-        session = create_session(bind=testbase.db)
+        session = create_session(bind=testing.db)
         query = session.query(tables.User)
         x = query.outerjoin(['orders', 'items']).filter(or_(tables.Order.c.order_id==None,tables.Item.c.item_id==2))
         print x.compile()
@@ -189,7 +190,7 @@ class RelationsTest(AssertMixin):
                 'items':relation(mapper(tables.Item, tables.orderitems))
             }))
         })
-        session = create_session(bind=testbase.db)
+        session = create_session(bind=testing.db)
         query = session.query(tables.User)
         x = query.outerjoin(['orders', 'items']).filter(or_(tables.Order.c.order_id==None,tables.Item.c.item_id==2)).count()
         assert x==2
@@ -199,7 +200,7 @@ class RelationsTest(AssertMixin):
                 'items':relation(mapper(tables.Item, tables.orderitems))
             }))
         })
-        session = create_session(bind=testbase.db)
+        session = create_session(bind=testing.db)
         query = session.query(tables.User)
         x = query.select_from(tables.users.outerjoin(tables.orders).outerjoin(tables.orderitems)).\
             filter(or_(tables.Order.c.order_id==None,tables.Item.c.item_id==2))
@@ -207,10 +208,10 @@ class RelationsTest(AssertMixin):
         self.assert_result(list(x), tables.User, *tables.user_result[1:3])
 
 
-class CaseSensitiveTest(PersistTest):
+class CaseSensitiveTest(TestBase):
     def setUpAll(self):
         global metadata, table1, table2
-        metadata = MetaData(testbase.db)
+        metadata = MetaData(testing.db)
         table1 = Table('Table1', metadata,
             Column('ID', Integer, primary_key=True),
             )
@@ -230,7 +231,7 @@ class CaseSensitiveTest(PersistTest):
         clear_mappers()
 
     def test_distinctcount(self):
-        q = create_session(bind=testbase.db).query(Obj1)
+        q = create_session(bind=testing.db).query(Obj1)
         assert q.count() == 4
         res = q.filter(and_(table1.c.ID==table2.c.T1ID,table2.c.T1ID==1))
         assert res.count() == 3
@@ -247,20 +248,18 @@ class SelfRefTest(ORMTest):
     def test_noautojoin(self):
         class T(object):pass
         mapper(T, t1, properties={'children':relation(T)})
-        sess = create_session(bind=testbase.db)
-        try:
-            sess.query(T).join('children').select_by(id=7)
-            assert False
-        except exceptions.InvalidRequestError, e:
-            assert str(e) == "Self-referential query on 'T.children (T)' property requires create_aliases=True argument.", str(e)
+        sess = create_session(bind=testing.db)
+        def go():
+            sess.query(T).join('children')
+        self.assertRaisesMessage(exceptions.InvalidRequestError, 
+            "Self-referential query on 'T\.children \(T\)' property requires aliased=True argument.", go)
 
-        try:
+        def go():
             sess.query(T).join(['children']).select_by(id=7)
-            assert False
-        except exceptions.InvalidRequestError, e:
-            assert str(e) == "Self-referential query on 'T.children (T)' property requires create_aliases=True argument.", str(e)
+        self.assertRaisesMessage(exceptions.InvalidRequestError, 
+            "Self-referential query on 'T\.children \(T\)' property requires aliased=True argument.", go)
 
 
 
 if __name__ == "__main__":
-    testbase.main()
+    testenv.main()
