@@ -1,4 +1,4 @@
-import testbase
+import testenv; testenv.configure_for_tests()
 from sqlalchemy import *
 from sqlalchemy.orm import *
 from sqlalchemy.ext.sessioncontext import SessionContext
@@ -24,12 +24,13 @@ class Port(object):
         self.name=name
         self.description = description
 
-class O2OTest(AssertMixin):
+class O2OTest(TestBase, AssertsExecutionResults):
+    @testing.uses_deprecated('SessionContext')
     def setUpAll(self):
         global jack, port, metadata, ctx
-        metadata = MetaData(testbase.db)
+        metadata = MetaData(testing.db)
         ctx = SessionContext(create_session)
-        jack = Table('jack', metadata, 
+        jack = Table('jack', metadata,
             Column('id', Integer, primary_key=True),
             #Column('room_id', Integer, ForeignKey("room.id")),
             Column('number', String(50)),
@@ -38,7 +39,7 @@ class O2OTest(AssertMixin):
         )
 
 
-        port = Table('port', metadata, 
+        port = Table('port', metadata,
             Column('id', Integer, primary_key=True),
             #Column('device_id', Integer, ForeignKey("device.id")),
             Column('name', String(30)),
@@ -52,12 +53,13 @@ class O2OTest(AssertMixin):
         clear_mappers()
     def tearDownAll(self):
         metadata.drop_all()
-            
+
+    @testing.uses_deprecated('SessionContext')
     def test1(self):
         mapper(Port, port, extension=ctx.mapper_extension)
         mapper(Jack, jack, order_by=[jack.c.number],properties = {
             'port': relation(Port, backref='jack', uselist=False, lazy=True),
-        }, extension=ctx.mapper_extension) 
+        }, extension=ctx.mapper_extension)
 
         j=Jack(number='101')
         p=Port(name='fa0/1')
@@ -87,5 +89,5 @@ class O2OTest(AssertMixin):
         ctx.current.delete(j)
         ctx.current.flush()
 
-if __name__ == "__main__":    
-    testbase.main()
+if __name__ == "__main__":
+    testenv.main()
