@@ -1,4 +1,4 @@
-import testbase
+import testenv; testenv.configure_for_tests()
 from sqlalchemy import *
 from sqlalchemy.orm import *
 from testlib import *
@@ -6,11 +6,11 @@ from testlib import *
 NUM = 500
 DIVISOR = 50
 
-meta = MetaData(testbase.db)
-items = Table('items', meta, 
+meta = MetaData(testing.db)
+items = Table('items', meta,
               Column('item_id', Integer, primary_key=True),
               Column('value', String(100)))
-subitems = Table('subitems', meta, 
+subitems = Table('subitems', meta,
                  Column('sub_id', Integer, primary_key=True),
                  Column('parent_id', Integer, ForeignKey('items.item_id')),
                  Column('value', String(100)))
@@ -33,12 +33,12 @@ def load():
             z = ((x-1) * DIVISOR) + y
             l.append({'sub_id':z,'value':'this is item #%d' % z, 'parent_id':x})
         #print l
-        subitems.insert().execute(*l)    
+        subitems.insert().execute(*l)
 
 @profiling.profiled('masseagerload', always=True, sort=['cumulative'])
 def masseagerload(session):
     query = session.query(Item)
-    l = query.select()
+    l = query.all()
     print "loaded ", len(l), " items each with ", len(l[0].subs), "subitems"
 
 def all():
