@@ -8,14 +8,16 @@ from testlib.schema import Table, Column
 from testlib.orm import mapper
 import testlib.testing as testing
 from testlib.testing import rowset
-from testlib.testing import PersistTest, AssertMixin, ORMTest, SQLCompileTest
+from testlib.testing import TestBase, AssertsExecutionResults, ORMTest, AssertsCompiledSQL, ComparesTables
 import testlib.profiling as profiling
 import testlib.engines as engines
+from testlib.compat import set, frozenset, sorted, _function_named
 
 
 __all__ = ('testing',
            'mapper',
            'Table', 'Column',
            'rowset',
-           'PersistTest', 'AssertMixin', 'ORMTest', 'SQLCompileTest',
-           'profiling', 'engines')
+           'TestBase', 'AssertsExecutionResults', 'ORMTest', 'AssertsCompiledSQL', 'ComparesTables',
+           'profiling', 'engines',
+           'set', 'frozenset', 'sorted', '_function_named')
