@@ -9,9 +9,10 @@ groups = [user, administrator]
 
 def cryptpw(password, salt=None):
     if salt is None:
-        salt = string.join([chr(random.randint(ord('a'), ord('z'))), chr(random.randint(ord('a'), ord('z')))],'')
+        salt = string.join([chr(random.randint(ord('a'), ord('z'))),
+                            chr(random.randint(ord('a'), ord('z')))],'')
     return sha(password + salt).hexdigest()
-    
+
 def checkpw(password, dbpw):
     return cryptpw(password, dbpw[:2]) == dbpw
 
@@ -32,4 +33,4 @@ class User(object):
     password = property(lambda s: None, _set_password)
 
     def checkpw(self, password):
-        return checkpw(password, self.crypt_password)
\ No newline at end of file
+        return checkpw(password, self.crypt_password)
